"""Dictionaries supporting conditions and phenotypes."""

import requests
import os

ontology_text = {
    '190400': 'TRIGEMINAL NEURALGIA',
    '207790': 'ARACHNOID CYSTS, INTRACRANIAL',
    'DOID:0014667': 'disease of metabolism',
    'DOID:0050589': 'inflammatory bowel disease',
    'DOID:0060041': 'autism spectrum disorder',
    'DOID:0060224': 'atrial fibrillation',
    'DOID:10652': "Alzheimer's disease",
    'DOID:12377': 'spinal muscular atrophy',
    'DOID:1287': 'cardiovascular system disease',
    'DOID:1826': 'epilepsy',
    'DOID:2841': 'asthma',
    'DOID:3393': 'coronary artery disease',
    'DOID:5419': 'schizophrenia',
    'DOID:5844': 'myocardial infarction',
    'DOID:630': 'genetic disease',
    'DOID:9744': 'type 1 diabetes mellitus',
    'MONDO:0000603': 'cardiovascular system disease',
    'MONDO:0001516': 'spinal muscular atrophy',
    'MONDO:0003847': 'Mendelian disorders',
    'MONDO:0004979': 'asthma',
    'MONDO:0004985': 'Schizophrenia and Bipolar Disorder cases and controls',
    'MONDO:0004995': 'cardiovascular disease',
    'MONDO:0005010': 'coronary artery disease',
    'MONDO:0005027': 'epilepsy',
    'MONDO:0005147': 'type 1 diabetes mellitus',
    'MONDO:0005258': 'autism spectrum disorder',
    'MONDO:0005265': 'inflammatory bowel disease',
    'MONDO:0005311': 'atherosclerosis',
    'MONDO:0019752': 'pediatric disease',
    'OMIM:100100': 'DIABETES INSIPIDUS, NEPHROGENIC, 2, AUTOSOMAL; NDI2',
    'OMIM:100300': 'ADAMS-OLIVER SYNDROME 1; AOS1',
    'OMIM:100800': 'ACHONDROPLASIA; ACH',
    'OMIM:101120': 'ACROCEPHALOPOLYSYNDACTYLY TYPE III',
    'OMIM:102200': 'PITUITARY ADENOMA 1, MULTIPLE TYPES; PITA1',
    'OMIM:102500': 'HAJDU-CHENEY SYNDROME; HJCYS',
    'OMIM:105400': 'AMYOTROPHIC LATERAL SCLEROSIS 1; ALS1',
    'OMIM:105650': 'DIAMOND-BLACKFAN ANEMIA 1; DBA1',
    'OMIM:105830': 'ANGELMAN SYNDROME; AS',
    'OMIM:107480': 'TOWNES-BROCKS SYNDROME 1; TBS1',
    'OMIM:107970': 'ARRHYTHMOGENIC RIGHT VENTRICULAR DYSPLASIA, FAMILIAL, 1; ARVD1',
    'OMIM:108120': 'ARTHROGRYPOSIS, DISTAL, TYPE 1A; DA1A',
    'OMIM:108145': 'ARTHROGRYPOSIS, DISTAL, TYPE 5; DA5',
    'OMIM:109730': 'AORTIC VALVE DISEASE 1; AOVD1',
    'OMIM:114300': 'ARTHROGRYPOSIS, DISTAL, TYPE 3; DA3',
    'OMIM:115195': 'CARDIOMYOPATHY, FAMILIAL HYPERTROPHIC, 2; CMH2',
    'OMIM:115196': 'CARDIOMYOPATHY, FAMILIAL HYPERTROPHIC, 3; CMH3',
    'OMIM:115197': 'CARDIOMYOPATHY, FAMILIAL HYPERTROPHIC, 4; CMH4',
    'OMIM:115200': 'CARDIOMYOPATHY, DILATED, 1A; CMD1A',
    'OMIM:117000': 'CENTRAL CORE DISEASE OF MUSCLE; CCD',
    'OMIM:117360': 'SPINOCEREBELLAR ATAXIA 29; SCA29',
    'OMIM:117550': 'SOTOS SYNDROME 1; SOTOS1',
    'OMIM:119530': 'OROFACIAL CLEFT 1; OFC1',
    'OMIM:122430': 'CORNEAL HYPESTHESIA WITH RETINAL ABNORMALITIES, SENSORINEURAL DEAFNESS, UNUSUAL FACIES, PERSISTENT DUCTUS ARTERIOSUS, AND MENTAL RETARDATION',
    'OMIM:122470': 'CORNELIA DE LANGE SYNDROME 1; CDLS1',
    'OMIM:125800': 'DIABETES INSIPIDUS, NEPHROGENIC, 2, AUTOSOMAL; NDI2',
    'OMIM:126600': 'DOYNE HONEYCOMB RETINAL DYSTROPHY; DHRD',
    'OMIM:130000': 'EHLERS-DANLOS SYNDROME, CLASSIC TYPE, 1; EDSCL1',
    'OMIM:130020': 'EHLERS-DANLOS SYNDROME, HYPERMOBILITY TYPE; EDSHMB',
    'OMIM:130600': 'ELLIPTOCYTOSIS 2; EL2',
    'OMIM:132900': 'AORTIC ANEURYSM, FAMILIAL THORACIC 4; AAT4',
    'OMIM:133180': 'ERYTHROLEUKEMIA, FAMILIAL, SUSCEPTIBILITY TO; FERLK',
    'OMIM:133700': 'EXOSTOSES, MULTIPLE, TYPE I; EXT1',
    'OMIM:133701': 'EXOSTOSES, MULTIPLE, TYPE II; EXT2',
    'OMIM:134200': 'FACIAL PALSY, FAMILIAL RECURRENT PERIPHERAL',
    'OMIM:135150': 'BIRT-HOGG-DUBE SYNDROME; BHD',
    'OMIM:135900': 'COFFIN-SIRIS SYNDROME 1; CSS1',
    'OMIM:136140': 'FLOATING-HARBOR SYNDROME; FLHS',
    'OMIM:136550': 'MACULAR DYSTROPHY, RETINAL, 1, NORTH CAROLINA TYPE; MCDR1',
    'OMIM:137580': 'GILLES DE LA TOURETTE SYNDROME; GTS',
    'OMIM:137920': 'RENAL CYSTS AND DIABETES SYNDROME; RCAD',
    'OMIM:139090': 'GRAY PLATELET SYNDROME; GPS',
    'OMIM:139210': 'MYHRE SYNDROME; MYHRS',
    'OMIM:142340': 'DIAPHRAGMATIC HERNIA, CONGENITAL',
    'OMIM:142690': 'ACNE INVERSA, FAMILIAL, 1; ACNINV1',
    'OMIM:143095': 'SPONDYLOEPIPHYSEAL DYSPLASIA WITH CONGENITAL JOINT DISLOCATIONS; SEDCJD',
    'OMIM:143465': 'ATTENTION DEFICIT-HYPERACTIVITY DISORDER; ADHD',
    'OMIM:143860': 'HYPERCHLORHIDROSIS, ISOLATED; HYCHL',
    'OMIM:146450': 'HYPOSPADIAS 3, AUTOSOMAL; HYSP3',
    'OMIM:147891': 'ISCHIOCOXOPODOPATELLAR SYNDROME WITH OR WITHOUT PULMONARY ARTERIAL HYPERTENSION; ICPPS',
    'OMIM:147920': 'KABUKI SYNDROME 1; KABUK1',
    'OMIM:148050': 'KBG SYNDROME; KBGS',
    'OMIM:149400': 'HYPEREKPLEXIA 1; HKPX1',
    'OMIM:151900': 'LIPOMATOSIS, MULTIPLE',
    'OMIM:152950': 'MICROCEPHALY WITH OR WITHOUT CHORIORETINOPATHY, LYMPHEDEMA, OR MENTAL RETARDATION; MCLMR',
    'OMIM:153670': 'BERNARD-SOULIER SYNDROME, TYPE A2, AUTOSOMAL DOMINANT; BSSA2',
    'OMIM:154400': 'ACROFACIAL DYSOSTOSIS 1, NAGER TYPE; AFD1',
    'OMIM:154700': 'MARFAN SYNDROME; MFS',
    'OMIM:154780': 'MARSHALL SYNDROME; MRSHS',
    'OMIM:155310': 'VISCERAL MYOPATHY; VSCM',
    'OMIM:157700': 'MITRAL VALVE PROLAPSE 1; MVP1',
    'OMIM:157980': 'MOMO SYNDROME',
    'OMIM:158590': 'NEURONOPATHY, DISTAL HEREDITARY MOTOR, TYPE IIA; HMN2A',
    'OMIM:158810': 'BETHLEM MYOPATHY 1; BTHLM1',
    'OMIM:159550': 'ATAXIA-PANCYTOPENIA SYNDROME; ATXPC',
    'OMIM:160150': 'MYOPATHY, CENTRONUCLEAR, 1; CNM1',
    'OMIM:160500': 'MYOPATHY, DISTAL, 1; MPD1',
    'OMIM:160800': 'MYOTONIA CONGENITA, AUTOSOMAL DOMINANT',
    'OMIM:160900': 'MYOTONIC DYSTROPHY 1; DM1',
    'OMIM:161800': 'NEMALINE MYOPATHY 3; NEM3',
    'OMIM:162091': 'SCHWANNOMATOSIS 1; SWNTS1',
    'OMIM:163950': 'NOONAN SYNDROME 1; NS1',
    'OMIM:164210': 'HEMIFACIAL MICROSOMIA; HFM',
    'OMIM:164230': 'OBSESSIVE-COMPULSIVE DISORDER; OCD',
    'OMIM:166000': 'ENCHONDROMATOSIS, MULTIPLE, OLLIER TYPE',
    'OMIM:168600': 'PARKINSON DISEASE, LATE-ONSET; PD',
    'OMIM:172870': 'PIGMENTED PARAVENOUS CHORIORETINAL ATROPHY; PPCRA',
    'OMIM:176450': 'CURRARINO SYNDROME',
    'OMIM:177850': 'PSEUDOXANTHOMA ELASTICUM, FORME FRUSTE',
    'OMIM:178110': 'CONTRACTURES, PTERYGIA, AND SPONDYLOCARPOTARSAL FUSION SYNDROME 1A; CPSFS1A',
    'OMIM:178500': 'PULMONARY FIBROSIS, IDIOPATHIC; IPF',
    'OMIM:180000': 'RETINAL ARTERIES, TORTUOSITY OF; RATOR',
    'OMIM:180100': 'RETINITIS PIGMENTOSA 1; RP1',
    'OMIM:180849': 'RUBINSTEIN-TAYBI SYNDROME 1; RSTS1',
    'OMIM:181430': 'SCAPULOPERONEAL MYOPATHY, MYH7-RELATED; SPMM',
    'OMIM:181700': 'SCLEROCORNEA, AUTOSOMAL DOMINANT',
    'OMIM:182212': 'SHPRINTZEN-GOLDBERG CRANIOSYNOSTOSIS SYNDROME; SGS',
    'OMIM:182601': 'SPASTIC PARAPLEGIA 4, AUTOSOMAL DOMINANT; SPG4',
    'OMIM:184255': 'SPONDYLOMETAPHYSEAL DYSPLASIA, CORNER FRACTURE TYPE; SMDCF',
    'OMIM:184900': 'STIFF SKIN SYNDROME; SSKS',
    'OMIM:185480': 'SUPRABULBAR PARESIS, CONGENITAL',
    'OMIM:186300': 'SYNDACTYLY, TYPE V; SDTY5',
    'OMIM:188000': 'THROMBOCYTOPENIA 2; THC2',
    'OMIM:190400': 'TRIGEMINAL NEURALGIA',
    'OMIM:191480': 'UNCOMBABLE HAIR SYNDROME 1; UHS1',
    'OMIM:191830': 'RENAL HYPODYSPLASIA/APLASIA 1; RHDA1',
    'OMIM:192350': 'VATER/VACTERL ASSOCIATION',
    'OMIM:192430': 'VELOCARDIOFACIAL SYNDROME; VCFS',
    'OMIM:192600': 'CARDIOMYOPATHY, FAMILIAL HYPERTROPHIC, 1; CMH1',
    'OMIM:203800': 'ALSTROM SYNDROME; ALMS',
    'OMIM:204200': 'CEROID LIPOFUSCINOSIS, NEURONAL, 3; CLN3',
    'OMIM:204500': 'CEROID LIPOFUSCINOSIS, NEURONAL, 2; CLN2',
    'OMIM:205100': 'AMYOTROPHIC LATERAL SCLEROSIS 2, JUVENILE; ALS2',
    'OMIM:206900': 'MICROPHTHALMIA, SYNDROMIC 3; MCOPS3',
    'OMIM:207790': 'ARACHNOID CYSTS, INTRACRANIAL',
    'OMIM:207950': 'CHIARI MALFORMATION TYPE II',
    'OMIM:208085': 'ARTHROGRYPOSIS, RENAL DYSFUNCTION, AND CHOLESTASIS 1; ARCS1',
    'OMIM:208230': 'PROGRESSIVE PSEUDORHEUMATOID DYSPLASIA; PPRD',
    'OMIM:208900': 'ATAXIA-TELANGIECTASIA; AT',
    'OMIM:209850': 'AUTISM',
    'OMIM:209900': 'BARDET-BIEDL SYNDROME 1; BBS1',
    'OMIM:210000': 'BEHR SYNDROME; BEHRS',
    'OMIM:210600': 'SECKEL SYNDROME 1; SCKL1',
    'OMIM:210720': 'MICROCEPHALIC OSTEODYSPLASTIC PRIMORDIAL DWARFISM, TYPE II; MOPD2',
    'OMIM:210900': 'BLOOM SYNDROME; BLM',
    'OMIM:211400': 'BRONCHIECTASIS WITH OR WITHOUT ELEVATED SWEAT CHLORIDE 1; BESC1',
    'OMIM:212065': 'CONGENITAL DISORDER OF GLYCOSYLATION, TYPE Ia; CDG1A',
    'OMIM:212066': 'CONGENITAL DISORDER OF GLYCOSYLATION, TYPE IIa; CDG2A',
    'OMIM:213300': 'Joubert syndrome 1',
    'OMIM:214500': 'CHEDIAK-HIGASHI SYNDROME; CHS',
    'OMIM:214800': 'CHARGE SYNDROME',
    'OMIM:215140': 'GREENBERG DYSPLASIA; GRBGD',
    'OMIM:215500': 'CHOROIDAL DYSTROPHY, CENTRAL AREOLAR, 1; CACD1',
    'OMIM:216340': 'YUNIS-VARON SYNDROME; YVS',
    'OMIM:216400': 'COCKAYNE SYNDROME A; CSA',
    'OMIM:216550': 'COHEN SYNDROME; COH1',
    'OMIM:217080': 'JALILI SYNDROME',
    'OMIM:217980': 'CORPUS CALLOSUM, AGENESIS OF, WITH FACIAL ANOMALIES AND ROBIN SEQUENCE',
    'OMIM:217990': 'CORPUS CALLOSUM, AGENESIS OF',
    'OMIM:218340': 'TEMTAMY SYNDROME; TEMTYS',
    'OMIM:219150': 'CUTIS LAXA, AUTOSOMAL RECESSIVE, TYPE IIIA; ARCL3A',
    'OMIM:219200': 'CUTIS LAXA, AUTOSOMAL RECESSIVE, TYPE IIA; ARCL2A',
    'OMIM:223370': 'DUBOWITZ SYNDROME',
    'OMIM:224050': 'CEREBELLAR ATAXIA, MENTAL RETARDATION, AND DYSEQUILIBRIUM SYNDROME 1; CAMRQ1',
    'OMIM:224690': 'MEIER-GORLIN SYNDROME 1; MGORS1',
    'OMIM:225753': 'PONTOCEREBELLAR HYPOPLASIA, TYPE 4; PCH4',
    'OMIM:225790': 'PROLIFERATIVE VASCULOPATHY AND HYDRANENCEPHALY-HYDROCEPHALY SYNDROME; PVHH',
    'OMIM:226750': 'KOHLSCHUTTER-TONZ SYNDROME; KTZS',
    'OMIM:227650': 'FANCONI ANEMIA, COMPLEMENTATION GROUP A; FANCA',
    'OMIM:228200': 'FEMUR-FIBULA-ULNA SYNDROME',
    'OMIM:229600': 'FRUCTOSE INTOLERANCE, HEREDITARY; HFI',
    'OMIM:230000': 'FUCOSIDOSIS',
    'OMIM:230500': 'GM1-GANGLIOSIDOSIS, TYPE I',
    'OMIM:231300': 'GLAUCOMA 3, PRIMARY CONGENITAL, A; GLC3A',
    'OMIM:232300': 'GLYCOGEN STORAGE DISEASE II; GSD2',
    'OMIM:232600': 'GLYCOGEN STORAGE DISEASE V; GSD5',
    'OMIM:234200': 'NEURODEGENERATION WITH BRAIN IRON ACCUMULATION 1; NBIA1',
    'OMIM:234500': 'HARTNUP DISORDER; HND',
    'OMIM:235400': 'HEMOLYTIC UREMIC SYNDROME, ATYPICAL, SUSCEPTIBILITY TO, 1; AHUS1',
    'OMIM:236000': 'LYMPHOMA, HODGKIN, CLASSIC; CHL',
    'OMIM:236600': 'HYDROCEPHALUS, CONGENITAL, 1; HYC1',
    'OMIM:236792': 'L-2-HYDROXYGLUTARIC ACIDURIA; L2HGA',
    'OMIM:242300': 'ICHTHYOSIS, CONGENITAL, AUTOSOMAL RECESSIVE 1; ARCI1',
    'OMIM:243310': 'BARAITSER-WINTER SYNDROME 1; BRWS1',
    'OMIM:244300': 'KAPUR-TORIELLO SYNDROME',
    'OMIM:244450': 'KAUFMAN OCULOCEREBROFACIAL SYNDROME; KOS',
    'OMIM:245200': 'KRABBE DISEASE',
    'OMIM:245359': 'missing',
    'OMIM:248200': 'STARGARDT DISEASE 1; STGD1',
    'OMIM:248700': 'MARDEN-WALKER SYNDROME; MWKS',
    'OMIM:249000': 'MECKEL SYNDROME, TYPE 1; MKS1',
    'OMIM:249900': 'METACHROMATIC LEUKODYSTROPHY DUE TO SAPOSIN B DEFICIENCY',
    'OMIM:250100': 'METACHROMATIC LEUKODYSTROPHY; MLD',
    'OMIM:250620': '3-HYDROXYISOBUTYRYL-CoA HYDROLASE DEFICIENCY; HIBCHD',
    'OMIM:251200': 'MICROCEPHALY 1, PRIMARY, AUTOSOMAL RECESSIVE; MCPH1',
    'OMIM:251260': 'NIJMEGEN BREAKAGE SYNDROME; NBS',
    'OMIM:251280': 'DIENCEPHALIC-MESENCEPHALIC JUNCTION DYSPLASIA SYNDROME 1; DMJDS1',
    'OMIM:251290': 'PSEUDO-TORCH SYNDROME 1; PTORCH1',
    'OMIM:251300': 'GALLOWAY-MOWAT SYNDROME 1; GAMOS1',
    'OMIM:252010': 'MITOCHONDRIAL COMPLEX I DEFICIENCY, NUCLEAR TYPE 1; MC1DN1',
    'OMIM:252160': 'MOLYBDENUM COFACTOR DEFICIENCY, COMPLEMENTATION GROUP B; MOCODB',
    'OMIM:252350': 'MOYAMOYA DISEASE 1; MYMY1',
    'OMIM:252920': 'MUCOPOLYSACCHARIDOSIS, TYPE IIIB; MPS3B',
    'OMIM:253250': 'MULIBREY NANISM',
    'OMIM:253600': 'MUSCULAR DYSTROPHY, LIMB-GIRDLE, AUTOSOMAL RECESSIVE 1; LGMDR1',
    'OMIM:253601': 'MUSCULAR DYSTROPHY, LIMB-GIRDLE, AUTOSOMAL RECESSIVE 2; LGMDR2',
    'OMIM:253700': 'MUSCULAR DYSTROPHY, LIMB-GIRDLE, AUTOSOMAL RECESSIVE 5; LGMDR5',
    'OMIM:254090': 'ULLRICH CONGENITAL MUSCULAR DYSTROPHY 1; UCMD1',
    'OMIM:254110': 'MUSCULAR DYSTROPHY, LIMB-GIRDLE, AUTOSOMAL RECESSIVE 8; LGMDR8',
    'OMIM:254300': 'MYASTHENIC SYNDROME, CONGENITAL, 10; CMS10',
    'OMIM:254780': 'MYOCLONIC EPILEPSY OF LAFORA',
    'OMIM:254940': 'CAREY-FINEMAN-ZITER SYNDROME; CFZS',
    'OMIM:255310': 'MYOPATHY, CONGENITAL, WITH FIBER-TYPE DISPROPORTION; CFTD',
    'OMIM:255320': 'MINICORE MYOPATHY WITH EXTERNAL OPHTHALMOPLEGIA',
    'OMIM:255800': 'SCHWARTZ-JAMPEL SYNDROME, TYPE 1; SJS1',
    'OMIM:255995': 'MYOPATHY, CONGENITAL, BAILEY-BLOCH; MYPBB',
    'OMIM:256000': 'LEIGH SYNDROME; LS',
    'OMIM:256300': 'NEPHROTIC SYNDROME, TYPE 1; NPHS1',
    'OMIM:256520': 'NEU-LAXOVA SYNDROME 1; NLS1',
    'OMIM:256600': 'NEURODEGENERATION WITH BRAIN IRON ACCUMULATION 2A; NBIA2A',
    'OMIM:256700': 'NEUROBLASTOMA, SUSCEPTIBILITY TO, 1; NBLST1',
    'OMIM:256730': 'CEROID LIPOFUSCINOSIS, NEURONAL, 1; CLN1',
    'OMIM:256731': 'CEROID LIPOFUSCINOSIS, NEURONAL, 5; CLN5',
    'OMIM:256800': 'INSENSITIVITY TO PAIN, CONGENITAL, WITH ANHIDROSIS; CIPA',
    'OMIM:256850': 'GIANT AXONAL NEUROPATHY 1, AUTOSOMAL RECESSIVE; GAN1',
    'OMIM:257220': 'NIEMANN-PICK DISEASE, TYPE C1; NPC1',
    'OMIM:257320': 'LISSENCEPHALY 2; LIS2',
    'OMIM:260400': 'SHWACHMAN-DIAMOND SYNDROME 1; SDS1',
    'OMIM:260565': 'PEHO SYNDROME; PEHO',
    'OMIM:261630': 'HYPERPHENYLALANINEMIA, BH4-DEFICIENT, C; HPABH4C',
    'OMIM:262500': 'LARON SYNDROME',
    'OMIM:263200': 'POLYCYSTIC KIDNEY DISEASE 4 WITH OR WITHOUT POLYCYSTIC LIVER DISEASE; PKD4',
    'OMIM:265000': 'MULTIPLE PTERYGIUM SYNDROME, ESCOBAR VARIANT; EVMPS',
    'OMIM:266100': 'EPILEPSY, PYRIDOXINE-DEPENDENT; EPD',
    'OMIM:266265': 'CONGENITAL DISORDER OF GLYCOSYLATION, TYPE IIc; CDG2C',
    'OMIM:266500': 'REFSUM DISEASE, CLASSIC',
    'OMIM:266510': 'PEROXISOME BIOGENESIS DISORDER 3B; PBD3B',
    'OMIM:266600': 'INFLAMMATORY BOWEL DISEASE (CROHN DISEASE) 1; IBD1',
    'OMIM:268800': 'SANDHOFF DISEASE',
    'OMIM:269200': 'AUTOIMMUNE POLYENDOCRINE SYNDROME, TYPE II; APS2',
    'OMIM:270200': 'SJOGREN-LARSSON SYNDROME; SLS',
    'OMIM:270800': 'SPASTIC PARAPLEGIA 5A, AUTOSOMAL RECESSIVE; SPG5A',
    'OMIM:271510': 'SPONDYLOEPIMETAPHYSEAL DYSPLASIA, SPONASTRIME TYPE; SEMDSP',
    'OMIM:271900': 'CANAVAN DISEASE',
    'OMIM:272800': 'TAY-SACHS DISEASE; TSD',
    'OMIM:273300': 'TESTICULAR GERM CELL TUMOR; TGCT',
    'OMIM:274000': 'THROMBOCYTOPENIA-ABSENT RADIUS SYNDROME; TAR',
    'OMIM:276901': 'USHER SYNDROME, TYPE IIA; USH2A',
    'OMIM:276950': 'VACTERL ASSOCIATION WITH HYDROCEPHALUS',
    'OMIM:277470': 'PONTOCEREBELLAR HYPOPLASIA, TYPE 2A; PCH2A',
    'OMIM:277590': 'WEAVER SYNDROME; WVS',
    'OMIM:300029': 'RETINITIS PIGMENTOSA 3; RP3',
    'OMIM:300049': 'PERIVENTRICULAR NODULAR HETEROTOPIA 1; PVNH1',
    'OMIM:300067': 'LISSENCEPHALY, X-LINKED, 1; LISX1',
    'OMIM:300088': 'DEVELOPMENTAL AND EPILEPTIC ENCEPHALOPATHY 9; DEE9',
    'OMIM:300100': 'ADRENOLEUKODYSTROPHY; ALD',
    'OMIM:300376': 'MUSCULAR DYSTROPHY, BECKER TYPE; BMD',
    'OMIM:300387': 'MENTAL RETARDATION, X-LINKED 63; MRX63',
    'OMIM:300476': 'CONE-ROD DYSTROPHY, X-LINKED, 3; CORDX3',
    'OMIM:300486': 'MENTAL RETARDATION, X-LINKED, WITH CEREBELLAR HYPOPLASIA AND DISTINCTIVE FACIAL APPEARANCE',
    'OMIM:300495': 'AUTISM, SUSCEPTIBILITY TO, X-LINKED 2; AUTSX2',
    'OMIM:300523': 'ALLAN-HERNDON-DUDLEY SYNDROME; AHDS',
    'OMIM:300534': 'MENTAL RETARDATION, X-LINKED, SYNDROMIC, CLAES-JENSEN TYPE; MRXSCJ',
    'OMIM:300695': 'SCAPULOPERONEAL MYOPATHY, X-LINKED DOMINANT; SPM',
    'OMIM:300749': 'MENTAL RETARDATION AND MICROCEPHALY WITH PONTINE AND CEREBELLAR HYPOPLASIA; MICPCH',
    'OMIM:300804': 'JOUBERT SYNDROME 10; JBTS10',
    'OMIM:300816': 'COMBINED OXIDATIVE PHOSPHORYLATION DEFICIENCY 6; COXPD6',
    'OMIM:300881': 'MOVED TO 615777',
    'OMIM:300882': 'CORNELIA DE LANGE SYNDROME 5; CDLS5',
    'OMIM:300894': 'NEURODEGENERATION WITH BRAIN IRON ACCUMULATION 5; NBIA5',
    'OMIM:300908': 'ANEMIA, NONSPHEROCYTIC HEMOLYTIC, DUE TO G6PD DEFICIENCY',
    'OMIM:300953': 'TRICHOTHIODYSTROPHY 5, NONPHOTOSENSITIVE; TTD5',
    'OMIM:300957': 'MENTAL RETARDATION, X-LINKED 12; MRX12',
    'OMIM:300958': 'INTELLECTUAL DEVELOPMENTAL DISORDER, X-LINKED, SYNDROMIC, SNIJDERS BLOK TYPE; MRXSSB',
    'OMIM:300966': 'MENTAL RETARDATION, X-LINKED, SYNDROMIC 33; MRXS33',
    'OMIM:300968': 'MENTAL RETARDATION, X-LINKED 99, SYNDROMIC, FEMALE-RESTRICTED; MRXS99F',
    'OMIM:300983': 'MENTAL RETARDATION, X-LINKED 104; MRX104',
    'OMIM:300986': 'MENTAL RETARDATION, X-LINKED, SYNDROMIC, BAIN TYPE; MRXSB',
    'OMIM:301050': 'ALPORT SYNDROME 1, X-LINKED; ATS1',
    'OMIM:301310': 'ANEMIA, SIDEROBLASTIC, AND SPINOCEREBELLAR ATAXIA; ASAT',
    'OMIM:304340': 'PETTIGREW SYNDROME; PGS',
    'OMIM:305000': 'DYSKERATOSIS CONGENITA, X-LINKED; DKCX',
    'OMIM:305100': 'ECTODERMAL DYSPLASIA 1, HYPOHIDROTIC, X-LINKED; XHED',
    'OMIM:308350': 'DEVELOPMENTAL AND EPILEPTIC ENCEPHALOPATHY 1; DEE1',
    'OMIM:309500': 'RENPENNING SYNDROME 1; RENS1',
    'OMIM:309541': 'METHYLMALONIC ACIDEMIA AND HOMOCYSTEINEMIA, cblX TYPE',
    'OMIM:309580': 'MENTAL RETARDATION-HYPOTONIC FACIES SYNDROME, X-LINKED, 1; MRXHF1',
    'OMIM:310200': 'Duchenne muscular dystrophy',
    'OMIM:310468': 'NEPHROLITHIASIS, X-LINKED RECESSIVE, WITH RENAL FAILURE; XRN',
    'OMIM:311070': 'CHARCOT-MARIE-TOOTH DISEASE, X-LINKED RECESSIVE, 5; CMTX5',
    'OMIM:312700': 'RETINOSCHISIS 1, X-LINKED, JUVENILE; RS1',
    'OMIM:312750': 'RETT SYNDROME; RTT',
    'OMIM:313850': 'THORACOABDOMINAL SYNDROME; THAS',
    'OMIM:314580': 'WIEACKER-WOLFF SYNDROME; WRWF',
    'OMIM:600059': 'RETINITIS PIGMENTOSA 13; RP13',
    'OMIM:600105': 'RETINITIS PIGMENTOSA 12; RP12',
    'OMIM:600118': 'WARBURG MICRO SYNDROME 1; WARBM1',
    'OMIM:600132': 'RETINITIS PIGMENTOSA 14; RP14',
    'OMIM:600138': 'RETINITIS PIGMENTOSA 11; RP11',
    'OMIM:600143': 'CEROID LIPOFUSCINOSIS, NEURONAL, 8; CLN8',
    'OMIM:600334': 'TIBIAL MUSCULAR DYSTROPHY, TARDIVE; TMD',
    'OMIM:600373': 'CODAS SYNDROME',
    'OMIM:600618': 'ETS VARIANT GENE 6; ETV6',
    'OMIM:600776': 'FRYNS MICROPHTHALMIA SYNDROME',
    'OMIM:600852': 'RETINITIS PIGMENTOSA 17; RP17',
    'OMIM:600920': 'VAN DEN ENDE-GUPTA SYNDROME; VDEGS',
    'OMIM:600977': 'CONE-ROD DYSTROPHY 5; CORD5',
    'OMIM:600995': 'NEPHROTIC SYNDROME, TYPE 2; NPHS2',
    'OMIM:601088': 'AYME-GRIPP SYNDROME; AYGRP',
    'OMIM:601144': 'BRUGADA SYNDROME 1; BRGDA1',
    'OMIM:601414': 'RETINITIS PIGMENTOSA 18; RP18',
    'OMIM:601419': 'MYOPATHY, MYOFIBRILLAR, 1; MFM1',
    'OMIM:601680': 'ARTHROGRYPOSIS, DISTAL, TYPE 2B1; DA2B1',
    'OMIM:601718': 'RETINITIS PIGMENTOSA 19; RP19',
    'OMIM:601780': 'CEROID LIPOFUSCINOSIS, NEURONAL, 6; CLN6',
    'OMIM:601813': 'EXUDATIVE VITREORETINOPATHY 4; EVR4',
    'OMIM:601815': 'PHOSPHOGLYCERATE DEHYDROGENASE DEFICIENCY; PHGDHD',
    'OMIM:601853': 'GOMEZ-LOPEZ-HERNANDEZ SYNDROME; GLHS',
    'OMIM:601954': 'MUSCULAR DYSTROPHY, LIMB-GIRDLE, AUTOSOMAL RECESSIVE 7; LGMDR7',
    'OMIM:602066': 'CONVULSIONS, FAMILIAL INFANTILE, WITH PAROXYSMAL CHOREOATHETOSIS; ICCA',
    'OMIM:602482': 'AXENFELD-RIEGER SYNDROME, TYPE 3; RIEG3',
    'OMIM:602535': 'MARSHALL-SMITH SYNDROME; MRSHSS',
    'OMIM:602579': 'CONGENITAL DISORDER OF GLYCOSYLATION, TYPE Ib; CDG1B',
    'OMIM:602772': 'RETINITIS PIGMENTOSA 25; RP25',
    'OMIM:603075': 'MACULAR DEGENERATION, AGE-RELATED, 1; ARMD1',
    'OMIM:603438': 'RADIOULNAR SYNOSTOSIS WITH MICROCEPHALY, SHORT STATURE, SCOLIOSIS, AND MENTAL RETARDATION',
    'OMIM:603511': 'MUSCULAR DYSTROPHY, LIMB-GIRDLE, AUTOSOMAL DOMINANT 1; LGMDD1',
    'OMIM:603896': 'LEUKOENCEPHALOPATHY WITH VANISHING WHITE MATTER; VWM',
    'OMIM:604004': 'MEGALENCEPHALIC LEUKOENCEPHALOPATHY WITH SUBCORTICAL CYSTS 1; MLC1',
    'OMIM:604116': 'CONE-ROD DYSTROPHY 3; CORD3',
    'OMIM:604273': 'MITOCHONDRIAL COMPLEX V (ATP SYNTHASE) DEFICIENCY, NUCLEAR TYPE 1; MC5DN1',
    'OMIM:604286': 'MUSCULAR DYSTROPHY, LIMB-GIRDLE, AUTOSOMAL RECESSIVE 4; LGMDR4',
    'OMIM:604317': 'MICROCEPHALY 2, PRIMARY, AUTOSOMAL RECESSIVE, WITH OR WITHOUT CORTICAL MALFORMATIONS; MCPH2',
    'OMIM:604360': 'SPASTIC PARAPLEGIA 11, AUTOSOMAL RECESSIVE; SPG11',
    'OMIM:604387': 'NEPHRONOPHTHISIS 3; NPHP3',
    'OMIM:604537': 'LEBER CONGENITAL AMAUROSIS 5; LCA5',
    'OMIM:604592': 'T CELL IMMUNE REGULATOR 1; TCIRG1',
    'OMIM:604715': 'ORTHOSTATIC INTOLERANCE',
    'OMIM:605130': 'WIEDEMANN-STEINER SYNDROME; WDSTS',
    'OMIM:605355': 'NEMALINE MYOPATHY 5; NEM5',
    'OMIM:605361': 'SPINOCEREBELLAR ATAXIA 14; SCA14',
    'OMIM:605407': 'SEGAWA SYNDROME, AUTOSOMAL RECESSIVE',
    'OMIM:605637': 'MYOPATHY, PROXIMAL, WITH OPHTHALMOPLEGIA; MYPOP',
    'OMIM:605670': 'LATE-ONSET RETINAL DEGENERATION; LORD',
    'OMIM:605820': 'NONAKA MYOPATHY; NM',
    'OMIM:605899': 'GLYCINE ENCEPHALOPATHY; GCE',
    'OMIM:606003': 'TRANSALDOLASE DEFICIENCY; TALDOD',
    'OMIM:606068': 'RETINITIS PIGMENTOSA 28; RP28',
    'OMIM:606157': 'PANTOTHENATE KINASE 2; PANK2',
    'OMIM:606170': 'GENITOPATELLAR SYNDROME; GTPTS',
    'OMIM:606232': 'PHELAN-MCDERMID SYNDROME; PHMDS',
    'OMIM:606519': 'PHACE ASSOCIATION',
    'OMIM:606777': 'GLUT1 DEFICIENCY SYNDROME 1; GLUT1DS1',
    'OMIM:606854': 'POLYMICROGYRIA, BILATERAL FRONTOPARIETAL; BFPP',
    'OMIM:606966': 'NEPHRONOPHTHISIS 4; NPHP4',
    'OMIM:606996': 'SENIOR-LOKEN SYNDROME 4; SLSN4',
    'OMIM:607015': 'HURLER-SCHEIE SYNDROME',
    'OMIM:607086': 'AORTIC ANEURYSM, FAMILIAL THORACIC 1; AAT1',
    'OMIM:607155': 'MUSCULAR DYSTROPHY-DYSTROGLYCANOPATHY (LIMB-GIRDLE), TYPE C, 5; MDDGC5',
    'OMIM:607196': 'MICROCEPHALY, AMISH TYPE; MCPHA',
    'OMIM:607225': 'SPASTIC PARALYSIS, INFANTILE-ONSET ASCENDING; IAHSP',
    'OMIM:607426': 'COENZYME Q10 DEFICIENCY, PRIMARY, 1; COQ10D1',
    'OMIM:607432': 'LISSENCEPHALY 1; LIS1',
    'OMIM:607483': 'THIAMINE METABOLISM DYSFUNCTION SYNDROME 2 (BIOTIN- OR THIAMINE-RESPONSIVE TYPE); THMD2',
    'OMIM:607595': 'MOVED TO 175780',
    'OMIM:607596': 'PONTOCEREBELLAR HYPOPLASIA, TYPE 1A; PCH1A',
    'OMIM:607834': 'ANXIETY',
    'OMIM:607855': 'MUSCULAR DYSTROPHY, CONGENITAL MEROSIN-DEFICIENT, 1A; MDC1A',
    'OMIM:608027': 'PONTOCEREBELLAR HYPOPLASIA, TYPE 3; PCH3',
    'OMIM:608091': 'JOUBERT SYNDROME 2; JBTS2',
    'OMIM:608099': 'MUSCULAR DYSTROPHY, LIMB-GIRDLE, AUTOSOMAL RECESSIVE 3; LGMDR3',
    'OMIM:608133': 'RETINITIS PIGMENTOSA 7; RP7',
    'OMIM:608358': 'MYOPATHY, MYOSIN STORAGE, AUTOSOMAL DOMINANT; MSMA',
    'OMIM:608380': 'RETINITIS PIGMENTOSA 26; RP26',
    'OMIM:608389': 'BRANCHIOOTIC SYNDROME 3; BOS3',
    'OMIM:608393': 'MICROCEPHALY 6, PRIMARY, AUTOSOMAL RECESSIVE; MCPH6',
    'OMIM:608415': 'PROLONGED ELECTRORETINAL RESPONSE SUPPRESSION; PERRS',
    'OMIM:608423': 'MUSCULAR DYSTROPHY, LIMB-GIRDLE, AUTOSOMAL DOMINANT 2; LGMDD2',
    'OMIM:608540': 'CONGENITAL DISORDER OF GLYCOSYLATION, TYPE Ik; CDG1K',
    'OMIM:608600': 'LIPODYSTROPHY, FAMILIAL PARTIAL, TYPE 1; FPLD1',
    'OMIM:608629': 'JOUBERT SYNDROME 3; JBTS3',
    'OMIM:608643': 'AROMATIC L-AMINO ACID DECARBOXYLASE DEFICIENCY',
    'OMIM:608716': 'MICROCEPHALY 5, PRIMARY, AUTOSOMAL RECESSIVE; MCPH5',
    'OMIM:608804': 'LEUKODYSTROPHY, HYPOMYELINATING, 2; HLD2',
    'OMIM:608807': 'MUSCULAR DYSTROPHY, LIMB-GIRDLE, AUTOSOMAL RECESSIVE 10; LGMDR10',
    'OMIM:608940': 'SPONDYLOMETAPHYSEAL DYSPLASIA WITH CONE-ROD DYSTROPHY; SMDCRD',
    'OMIM:609033': 'POSTERIOR COLUMN ATAXIA WITH RETINITIS PIGMENTOSA; AXPC1',
    'OMIM:609056': 'SALT AND PEPPER DEVELOPMENTAL REGRESSION SYNDROME; SPDRS',
    'OMIM:609192': 'LOEYS-DIETZ SYNDROME 1; LDS1',
    'OMIM:609195': 'SPASTIC PARAPLEGIA 26, AUTOSOMAL RECESSIVE; SPG26',
    'OMIM:609200': 'MYOPATHY, MYOFIBRILLAR, 3; MFM3',
    'OMIM:609260': 'CHARCOT-MARIE-TOOTH DISEASE, AXONAL, AUTOSOMAL DOMINANT, TYPE 2A2A; CMT2A2A',
    'OMIM:609270': 'SPINOCEREBELLAR ATAXIA, AUTOSOMAL RECESSIVE 7; SCAR7',
    'OMIM:609304': 'DEVELOPMENTAL AND EPILEPTIC ENCEPHALOPATHY 3; DEE3',
    'OMIM:609524': 'MYOPATHY, MYOFIBRILLAR, 5; MFM5',
    'OMIM:609528': 'CEREBRAL DYSGENESIS, NEUROPATHY, ICHTHYOSIS, AND PALMOPLANTAR KERATODERMA SYNDROME',
    'OMIM:609560': 'MITOCHONDRIAL DNA DEPLETION SYNDROME 2 (MYOPATHIC TYPE); MTDPS2',
    'OMIM:609923': 'RETINITIS PIGMENTOSA 31; RP31',
    'OMIM:610019': 'CATARACT 18; CTRCT18',
    'OMIM:610031': 'CORTICAL DYSPLASIA, COMPLEX, WITH OTHER BRAIN MALFORMATIONS 7; CDCBM7',
    'OMIM:610127': 'CEROID LIPOFUSCINOSIS, NEURONAL, 10; CLN10',
    'OMIM:610181': 'AICARDI-GOUTIERES SYNDROME 2; AGS2',
    'OMIM:610188': 'JOUBERT SYNDROME 5; JBTS5',
    'OMIM:610217': 'NEURODEGENERATION WITH BRAIN IRON ACCUMULATION 2B; NBIA2B',
    'OMIM:610246': 'SPINOCEREBELLAR ATAXIA 28; SCA28',
    'OMIM:610329': 'AICARDI-GOUTIERES SYNDROME 3; AGS3',
    'OMIM:610333': 'AICARDI-GOUTIERES SYNDROME 4; AGS4',
    'OMIM:610359': 'RETINITIS PIGMENTOSA 33; RP33',
    'OMIM:610443': 'KOOLEN-DE VRIES SYNDROME; KDVS',
    'OMIM:610444': 'NIGHT BLINDNESS, CONGENITAL STATIONARY, AUTOSOMAL DOMINANT 3; CSNBAD3',
    'OMIM:610489': 'PIGMENTED NODULAR ADRENOCORTICAL DISEASE, PRIMARY, 1; PPNAD1',
    'OMIM:610532': 'LEUKODYSTROPHY, HYPOMYELINATING, 5; HLD5',
    'OMIM:610688': 'JOUBERT SYNDROME 6; JBTS6',
    'OMIM:610725': 'NEPHROTIC SYNDROME, TYPE 3; NPHS3',
    'OMIM:610951': 'CEROID LIPOFUSCINOSIS, NEURONAL, 7; CLN7',
    'OMIM:611091': 'MENTAL RETARDATION, AUTOSOMAL RECESSIVE 5; MRT5',
    'OMIM:611302': 'SPASTIC ATAXIA 2, AUTOSOMAL RECESSIVE; SPAX2',
    'OMIM:611307': 'MUSCULAR DYSTROPHY, LIMB-GIRDLE, AUTOSOMAL RECESSIVE 12; LGMDR12',
    'OMIM:611560': 'JOUBERT SYNDROME 7; JBTS7',
    'OMIM:611603': 'LISSENCEPHALY 3; LIS3',
    'OMIM:611705': 'SALIH MYOPATHY; SALMY',
    'OMIM:611721': 'COMBINED SAPOSIN DEFICIENCY',
    'OMIM:611726': 'EPILEPSY, PROGRESSIVE MYOCLONIC, 3, WITH OR WITHOUT INTRACELLULAR INCLUSIONS; EPM3',
    'OMIM:611755': 'LEBER CONGENITAL AMAUROSIS 10; LCA10',
    'OMIM:612015': 'CONGENITAL DISORDER OF GLYCOSYLATION, TYPE In; CDG1N',
    'OMIM:612095': 'RETINITIS PIGMENTOSA 41; RP41',
    'OMIM:612126': 'GLUT1 DEFICIENCY SYNDROME 2; GLUT1DS2',
    'OMIM:612199': 'CEREBRORETINAL MICROANGIOPATHY WITH CALCIFICATIONS AND CYSTS 1; CRMCC1',
    'OMIM:612280': 'FUCOSIDASE, ALPHA-L, 1; FUCA1',
    'OMIM:612285': 'JOUBERT SYNDROME 9; JBTS9',
    'OMIM:612337': 'MENTAL RETARDATION, AUTOSOMAL DOMINANT 22; MRD22',
    'OMIM:612370': 'HYPOGONADOTROPIC HYPOGONADISM 5 WITH OR WITHOUT ANOSMIA; HH5',
    'OMIM:612438': 'LEUKODYSTROPHY, HYPOMYELINATING, 6; HLD6',
    'OMIM:612528': 'DIAMOND-BLACKFAN ANEMIA 5; DBA5',
    'OMIM:612621': 'MENTAL RETARDATION, AUTOSOMAL DOMINANT 5; MRD5',
    'OMIM:612712': 'LEBER CONGENITAL AMAUROSIS 13; LCA13',
    'OMIM:612716': 'DYSTONIA, DOPA-RESPONSIVE, DUE TO SEPIAPTERIN REDUCTASE DEFICIENCY',
    'OMIM:612780': 'SEIZURES, SENSORINEURAL DEAFNESS, ATAXIA, MENTAL RETARDATION, AND ELECTROLYTE IMBALANCE; SESAMES',
    'OMIM:612863': 'CHROMOSOME 6q24-q25 DELETION SYNDROME',
    'OMIM:612936': 'SPASTIC PARAPLEGIA 50, AUTOSOMAL RECESSIVE; SPG50',
    'OMIM:612943': 'RETINITIS PIGMENTOSA 42; RP42',
    'OMIM:612951': 'LEUKOENCEPHALOPATHY, CYSTIC, WITHOUT MEGALENCEPHALY',
    'OMIM:613065': 'LEUKEMIA, ACUTE LYMPHOBLASTIC; ALL',
    'OMIM:613075': 'MACS SYNDROME',
    'OMIM:613091': 'SHORT-RIB THORACIC DYSPLASIA 3 WITH OR WITHOUT POLYDACTYLY; SRTD3',
    'OMIM:613155': 'MUSCULAR DYSTROPHY-DYSTROGLYCANOPATHY (CONGENITAL WITH IMPAIRED INTELLECTUAL IMPAIRMENT), TYPE B, 1; MDDGB1',
    'OMIM:613156': 'MUSCULAR DYSTROPHY-DYSTROGLYCANOPATHY (CONGENITAL WITH IMPAIRED INTELLECTUAL DEVELOPMENT), TYPE B, 2; MDDGB2',
    'OMIM:613158': 'MUSCULAR DYSTROPHY-DYSTROGLYCANOPATHY (LIMB-GIRDLE), TYPE C, 2; MDDGC2',
    'OMIM:613172': 'CARDIOMYOPATHY, DILATED, 1DD; CMD1DD',
    'OMIM:613192': 'MENTAL RETARDATION, AUTOSOMAL RECESSIVE 13; MRT13',
    'OMIM:613205': 'MUSCULAR DYSTROPHY, CONGENITAL, LMNA-RELATED',
    'OMIM:613216': 'NIGHT BLINDNESS, CONGENITAL STATIONARY, TYPE 1C; CSNB1C',
    'OMIM:613217': 'DIARRHEA 5, WITH TUFTING ENTEROPATHY, CONGENITAL; DIAR5',
    'OMIM:613229': 'TRICHOTILLOMANIA; TTM',
    'OMIM:613402': 'MICROCEPHALY, SEIZURES, AND DEVELOPMENTAL DELAY; MCSZ',
    'OMIM:613428': 'RETINITIS PIGMENTOSA 54; RP54',
    'OMIM:613454': 'RETT SYNDROME, CONGENITAL VARIANT',
    'OMIM:613550': 'NEPHRONOPHTHISIS 11; NPHP11',
    'OMIM:613561': 'MYOPATHY, LACTIC ACIDOSIS, AND SIDEROBLASTIC ANEMIA 2; MLASA2',
    'OMIM:613587': 'OCCULT MACULAR DYSTROPHY; OCMD',
    'OMIM:613612': 'CONGENITAL DISORDER OF GLYCOSYLATION, TYPE IIi; CDG2I',
    'OMIM:613660': 'CONE-ROD DYSTROPHY 15; CORD15',
    'OMIM:613668': 'MICROCEPHALY, POSTNATAL PROGRESSIVE, WITH SEIZURES AND BRAIN ATROPHY',
    'OMIM:613680': 'BEAULIEU-BOYCOTT-INNES SYNDROME; BBIS',
    'OMIM:613684': 'RUBINSTEIN-TAYBI SYNDROME 2; RSTS2',
    'OMIM:613690': 'CARDIOMYOPATHY, FAMILIAL HYPERTROPHIC, 7; CMH7',
    'OMIM:613706': 'NOONAN SYNDROME 7; NS7',
    'OMIM:613722': 'DEVELOPMENTAL AND EPILEPTIC ENCEPHALOPATHY 12; DEE12',
    'OMIM:613731': 'RETINITIS PIGMENTOSA 4; RP4',
    'OMIM:613744': 'SPASTIC PARAPLEGIA 51, AUTOSOMAL RECESSIVE; SPG51',
    'OMIM:613750': 'RETINITIS PIGMENTOSA 27; RP27',
    'OMIM:613756': 'RETINITIS PIGMENTOSA 49; RP49',
    'OMIM:613767': 'RETINITIS PIGMENTOSA 45; RP45',
    'OMIM:613794': 'RETINITIS PIGMENTOSA 20; RP20',
    'OMIM:613801': 'Retinitis pigmentosa-40',
    'OMIM:613809': 'RETINITIS PIGMENTOSA 39; RP39',
    'OMIM:613810': 'RETINITIS PIGMENTOSA 43; RP43',
    'OMIM:613811': 'PONTOCEREBELLAR HYPOPLASIA, TYPE 2D; PCH2D',
    'OMIM:613820': 'NEPHRONOPHTHISIS 12; NPHP12',
    'OMIM:613826': 'Leber congenital amaurosis 6',
    'OMIM:613835': 'LEBER CONGENITAL AMAUROSIS 8; LCA8',
    'OMIM:613861': 'RETINITIS PIGMENTOSA 59; RP59',
    'OMIM:613862': 'RETINITIS PIGMENTOSA 38; RP38',
    'OMIM:613982': 'OSTEOGENESIS IMPERFECTA, TYPE VI; OI6',
    'OMIM:614034': 'HEME OXYGENASE 1 DEFICIENCY; HMOX1D',
    'OMIM:614065': 'MYOPATHY, DISTAL, 4; MPD4',
    'OMIM:614173': 'JOUBERT SYNDROME 13; JBTS13',
    'OMIM:614180': 'RETINITIS PIGMENTOSA 61; RP61',
    'OMIM:614181': 'RETINITIS PIGMENTOSA 62; RP62',
    'OMIM:614199': 'NEPHROTIC SYNDROME, TYPE 5, WITH OR WITHOUT OCULAR ABNORMALITIES; NPHS5',
    'OMIM:614202': 'RAFIQ SYNDROME; RAFQS',
    'OMIM:614219': 'ADAMS-OLIVER SYNDROME 2; AOS2',
    'OMIM:614225': 'WARBURG MICRO SYNDROME 2; WARBM2',
    'OMIM:614254': 'NEURODEVELOPMENTAL DISORDER WITH OR WITHOUT HYPERKINETIC MOVEMENTS AND SEIZURES, AUTOSOMAL DOMINANT; NDHMSD',
    'OMIM:614255': 'NESCAV SYNDROME; NESCAVS',
    'OMIM:614299': 'MULTIPLE MITOCHONDRIAL DYSFUNCTIONS SYNDROME 2 WITH HYPERGLYCINEMIA; MMDS2',
    'OMIM:614381': 'LEUKODYSTROPHY, HYPOMYELINATING, 8, WITH OR WITHOUT OLIGODONTIA AND/OR HYPOGONADOTROPIC HYPOGONADISM; HLD8',
    'OMIM:614416': 'RADIOHUMERAL FUSIONS WITH OTHER SKELETAL AND CRANIOFACIAL ANOMALIES; RHFCA',
    'OMIM:614424': 'JOUBERT SYNDROME 14; JBTS14',
    'OMIM:614500': 'CONE-ROD DYSTROPHY 16; CORD16',
    'OMIM:614557': 'EHLERS-DANLOS SYNDROME, KYPHOSCOLIOTIC TYPE, 2; EDSKSCL2',
    'OMIM:614561': 'LEUKOENCEPHALOPATHY, BRAIN CALCIFICATIONS, AND CYSTS; LCC',
    'OMIM:614569': 'MULTIPLE ENCHONDROMATOSIS, MAFFUCCI TYPE',
    'OMIM:614575': 'CEREBELLAR ATAXIA, NEUROPATHY, AND VESTIBULAR AREFLEXIA SYNDROME; CANVAS',
    'OMIM:614607': 'COFFIN-SIRIS SYNDROME 2; CSS2',
    'OMIM:614615': 'JOUBERT SYNDROME 17; JBTS17',
    'OMIM:614669': 'AURICULOCONDYLAR SYNDROME 2; ARCND2',
    'OMIM:614678': 'PONTOCEREBELLAR HYPOPLASIA, TYPE 1B; PCH1B',
    'OMIM:614744': 'FACIAL PARESIS, HEREDITARY CONGENITAL, 3; HCFP3',
    'OMIM:614753': 'SOTOS SYNDROME 2; SOTOS2',
    'OMIM:614816': 'LOEYS-DIETZ SYNDROME 4; LDS4',
    'OMIM:614831': 'SPINOCEREBELLAR ATAXIA, AUTOSOMAL RECESSIVE 13; SCAR13',
    'OMIM:614833': 'MICROCEPHALY, SHORT STATURE, AND POLYMICROGYRIA WITH OR WITHOUT SEIZURES; MSSP',
    'OMIM:614844': 'NEPHRONOPHTHISIS 14; NPHP14',
    'OMIM:614875': 'METAPHYSEAL ENCHONDROMATOSIS WITH D-2-HYDROXYGLUTARIC ACIDURIA',
    'OMIM:614876': 'PEROXISOME BIOGENESIS DISORDER 8A (ZELLWEGER); PBD8A',
    'OMIM:614877': 'PEROXISOME BIOGENESIS DISORDER 8B; PBD8B',
    'OMIM:614921': 'CONGENITAL DISORDER OF GLYCOSYLATION, TYPE It; CDG1T',
    'OMIM:614932': 'COMBINED OXIDATIVE PHOSPHORYLATION DEFICIENCY 13; COXPD13',
    'OMIM:614945': 'DEAFNESS, AUTOSOMAL RECESSIVE 18B; DFNB18B',
    'OMIM:615030': 'SPASTIC PARAPLEGIA 56, AUTOSOMAL RECESSIVE; SPG56',
    'OMIM:615031': 'SPASTIC PARAPLEGIA 49, AUTOSOMAL RECESSIVE; SPG49',
    'OMIM:615032': 'AUTISM, SUSCEPTIBILITY TO, 18; AUTS18',
    'OMIM:615065': 'ARTHROGRYPOSIS, DISTAL, TYPE 5D; DA5D',
    'OMIM:615071': 'ALAZAMI SYNDROME; ALAZS',
    'OMIM:615095': 'MICROCEPHALY 10, PRIMARY, AUTOSOMAL RECESSIVE; MCPH10',
    'OMIM:615145': 'MICROPHTHALMIA, ISOLATED, WITH COLOBOMA 9; MCOPCB9',
    'OMIM:615181': 'MUSCULAR DYSTROPHY-DYSTROGLYCANOPATHY (CONGENITAL WITH BRAIN AND EYE ANOMALIES), TYPE A, 11; MDDGA11',
    'OMIM:615190': 'DYSKERATOSIS CONGENITA, AUTOSOMAL RECESSIVE 5; DKCB5',
    'OMIM:615191': 'LISSENCEPHALY 5; LIS5',
    'OMIM:615268': 'CEREBELLAR ATAXIA, MENTAL RETARDATION, AND DYSEQUILIBRIUM SYNDROME 4; CAMRQ4',
    'OMIM:615286': 'MENTAL RETARDATION, AUTOSOMAL RECESSIVE 36; MRT36',
    'OMIM:615352': 'MUSCULAR DYSTROPHY-DYSTROGLYCANOPATHY (LIMB-GIRDLE), TYPE C, 14; MDDGC14',
    'OMIM:615369': 'DEVELOPMENTAL AND EPILEPTIC ENCEPHALOPATHY 94; DEE94',
    'OMIM:615411': 'CORTICAL DYSPLASIA, COMPLEX, WITH OTHER BRAIN MALFORMATIONS 3; CDCBM3',
    'OMIM:615412': 'CORTICAL DYSPLASIA, COMPLEX, WITH OTHER BRAIN MALFORMATIONS 4; CDCBM4',
    'OMIM:615419': 'HYPOTONIA, INFANTILE, WITH PSYCHOMOTOR RETARDATION AND CHARACTERISTIC FACIES 1; IHPRF1',
    'OMIM:615471': 'MITOCHONDRIAL DNA DEPLETION SYNDROME 13 (ENCEPHALOMYOPATHIC TYPE); MTDPS13',
    'OMIM:615476': 'DEVELOPMENTAL AND EPILEPTIC ENCEPHALOPATHY 18; DEE18',
    'OMIM:615485': 'BAINBRIDGE-ROPERS SYNDROME; BRPS',
    'OMIM:615502': 'MENTAL RETARDATION, AUTOSOMAL DOMINANT 21; MRD21',
    'OMIM:615510': 'ALACRIMA, ACHALASIA, AND MENTAL RETARDATION SYNDROME; AAMR',
    'OMIM:615546': 'VAN MALDERGEM SYNDROME 2; VMLDS2',
    'OMIM:615574': 'ASPARAGINE SYNTHETASE DEFICIENCY; ASNSD',
    'OMIM:615670': 'SCHWANNOMATOSIS 2; SWNTS2',
    'OMIM:615681': 'SPASTIC PARAPLEGIA 62, AUTOSOMAL RECESSIVE; SPG62',
    'OMIM:615688': 'VASCULITIS, AUTOINFLAMMATION, IMMUNODEFICIENCY, AND HEMATOLOGIC DEFECTS SYNDROME; VAIHS',
    'OMIM:615715': 'BONE MARROW FAILURE SYNDROME 2; BMFS2',
    'OMIM:615716': 'HYPERPHOSPHATASIA WITH MENTAL RETARDATION SYNDROME 4; HPMRS4',
    'OMIM:615760': 'MICROCEPHALY, PROGRESSIVE, WITH SEIZURES AND CEREBRAL AND CEREBELLAR ATROPHY; MSCCA',
    'OMIM:615763': 'CORTICAL DYSPLASIA, COMPLEX, WITH OTHER BRAIN MALFORMATIONS 5; CDCBM5',
    'OMIM:615780': 'Retinitis pigmentosa 69',
    'OMIM:615809': 'PONTOCEREBELLAR HYPOPLASIA, TYPE 9; PCH9',
    'OMIM:615829': 'XIA-GIBBS SYNDROME; XIGIS',
    'OMIM:615834': 'MENTAL RETARDATION, AUTOSOMAL DOMINANT 26; MRD26',
    'OMIM:615846': 'AICARDI-GOUTIERES SYNDROME 7; AGS7',
    'OMIM:615866': 'COFFIN-SIRIS SYNDROME 9; CSS9',
    'OMIM:615877': 'MICROPHTHALMIA/COLOBOMA AND SKELETAL DYSPLASIA SYNDROME; MCSKS',
    'OMIM:615905': 'DEVELOPMENTAL AND EPILEPTIC ENCEPHALOPATHY 25, WITH AMELOGENESIS IMPERFECTA; DEE25',
    'OMIM:615909': 'DIAMOND-BLACKFAN ANEMIA 13; DBA13',
    'OMIM:615948': 'OROFACIODIGITAL SYNDROME XIV; OFD14',
    'OMIM:615959': 'MYOPATHY, CENTRONUCLEAR, 5; CNM5',
    'OMIM:615960': 'PORETTI-BOLTSHAUSER SYNDROME; PTBHS',
    'OMIM:615973': 'CONE-ROD DYSTROPHY 20; CORD20',
    'OMIM:615981': 'BARDET-BIEDL SYNDROME 2; BBS2',
    'OMIM:615982': 'BARDET-BIEDL SYNDROME 4; BBS4',
    'OMIM:615983': 'BARDET-BIEDL SYNDROME 5; BBS5',
    'OMIM:615993': 'BARDET-BIEDL SYNDROME 16; BBS16',
    'OMIM:616051': 'MICROCEPHALY 13, PRIMARY, AUTOSOMAL RECESSIVE; MCPH13',
    'OMIM:616056': 'DEVELOPMENTAL AND EPILEPTIC ENCEPHALOPATHY 26; DEE26',
    'OMIM:616079': 'RETINAL DYSTROPHY WITH INNER RETINAL DYSFUNCTION AND GANGLION CELL ABNORMALITIES; RDGCA',
    'OMIM:616081': 'PONTOCEREBELLAR HYPOPLASIA, TYPE 1C; PCH1C',
    'OMIM:616094': 'MUSCULAR DYSTROPHY-DYSTROGLYCANOPATHY (LIMB-GIRDLE), TYPE C, 12; MDDGC12',
    'OMIM:616100': 'AUTOIMMUNE LYMPHOPROLIFERATIVE SYNDROME, TYPE V; ALPS5',
    'OMIM:616127': 'SPINOCEREBELLAR ATAXIA, AUTOSOMAL RECESSIVE 17; SCAR17',
    'OMIM:616140': 'LEUKODYSTROPHY, HYPOMYELINATING, 9; HLD9',
    'OMIM:616155': 'CHARCOT-MARIE-TOOTH DISEASE, AXONAL, TYPE 2S; CMT2S',
    'OMIM:616171': 'MICROCEPHALY AND CHORIORETINOPATHY, AUTOSOMAL RECESSIVE, 2; MCCRP2',
    'OMIM:616199': 'POLYGLUCOSAN BODY MYOPATHY 2; PGBM2',
    'OMIM:616211': 'DEVELOPMENTAL AND EPILEPTIC ENCEPHALOPATHY 28; DEE28',
    'OMIM:616220': 'FOCAL SEGMENTAL GLOMERULOSCLEROSIS 9; FSGS9',
    'OMIM:616263': 'NEUROLOGIC, ENDOCRINE, AND PANCREATIC DISEASE, MULTISYSTEM, INFANTILE-ONSET; IMNEPD',
    'OMIM:616266': 'CONGENITAL CONTRACTURES OF THE LIMBS AND FACE, HYPOTONIA, AND DEVELOPMENTAL DELAY; CLIFAHDD',
    'OMIM:616271': '3-METHYLGLUTACONIC ACIDURIA, TYPE VII; MGCA7',
    'OMIM:616277': 'MITOCHONDRIAL SHORT-CHAIN ENOYL-CoA HYDRATASE 1 DEFICIENCY; ECHS1D',
    'OMIM:616281': 'NEURODEVELOPMENTAL DISORDER WITH SPASTIC PARAPLEGIA AND MICROCEPHALY; NEDSPM',
    'OMIM:616321': 'MYASTHENIC SYNDROME, CONGENITAL, 3A, SLOW-CHANNEL; CMS3A',
    'OMIM:616354': 'SPINOCEREBELLAR ATAXIA, AUTOSOMAL RECESSIVE 20; SCAR20',
    'OMIM:616390': 'TRICHOTHIODYSTROPHY 2, PHOTOSENSITIVE; TTD2',
    'OMIM:616394': 'RETINITIS PIGMENTOSA 71; RP71',
    'OMIM:616420': 'LEUKODYSTROPHY, HYPOMYELINATING, 10; HLD10',
    'OMIM:616421': 'MYOCLONIC-ATONIC EPILEPSY; MAE',
    'OMIM:616471': 'BETHLEM MYOPATHY 2; BTHLM2',
    'OMIM:616490': 'JOUBERT SYNDROME 23; JBTS23',
    'OMIM:616531': 'POLYMICROGYRIA, PERISYLVIAN, WITH CEREBELLAR HYPOPLASIA AND ARTHROGRYPOSIS; PMGYCHA',
    'OMIM:616544': 'RETINITIS PIGMENTOSA 73; RP73',
    'OMIM:616562': 'RETINITIS PIGMENTOSA 74; RP74',
    'OMIM:616564': 'NOONAN SYNDROME 10; NS10',
    'OMIM:616576': 'IMMUNODEFICIENCY, COMMON VARIABLE, 12, WITH AUTOIMMUNITY; CVID12',
    'OMIM:616577': 'EPILEPSY, HEARING LOSS, AND MENTAL RETARDATION SYNDROME; EHLMRS',
    'OMIM:616580': 'AU-KLINE SYNDROME; AUKS',
    'OMIM:616632': 'SEIZURES, CORTICAL BLINDNESS, AND MICROCEPHALY SYNDROME; SCBMS',
    'OMIM:616647': 'DEVELOPMENTAL AND EPILEPTIC ENCEPHALOPATHY 35; DEE35',
    'OMIM:616649': 'SPHEROCYTOSIS, TYPE 2; SPH2',
    'OMIM:616651': 'ROIFMAN SYNDROME; RFMN',
    'OMIM:616654': 'JOUBERT SYNDROME 24; JBTS24',
    'OMIM:616657': 'SPASTIC TETRAPLEGIA, THIN CORPUS CALLOSUM, AND PROGRESSIVE MICROCEPHALY; SPATCCM',
    'OMIM:616668': 'CHARCOT-MARIE-TOOTH DISEASE, AXONAL, TYPE 2X; CMT2X',
    'OMIM:616683': 'LEUKODYSTROPHY, HYPOMYELINATING, 12; HLD12',
    'OMIM:616721': 'CONGENITAL DISORDER OF GLYCOSYLATION, TYPE IIn; CDG2N',
    'OMIM:616723': 'SPONDYLOEPIMETAPHYSEAL DYSPLASIA, FADEN-ALKURAYA TYPE; SEMDFA',
    'OMIM:616732': 'OPTIC ATROPHY 10 WITH OR WITHOUT ATAXIA, MENTAL RETARDATION, AND SEIZURES; OPA10',
    'OMIM:616738': 'RADIOULNAR SYNOSTOSIS WITH AMEGAKARYOCYTIC THROMBOCYTOPENIA 2; RUSAT2',
    'OMIM:616740': 'IMMUNODEFICIENCY 46; IMD46',
    'OMIM:616789': 'MENTAL RETARDATION AND DISTINCTIVE FACIAL FEATURES WITH OR WITHOUT CARDIAC DEFECTS; MRFACD',
    'OMIM:616801': 'HYPOTONIA, INFANTILE, WITH PSYCHOMOTOR RETARDATION AND CHARACTERISTIC FACIES 2; IHPRF2',
    'OMIM:616875': 'CEREBELLAR ATROPHY, VISUAL IMPAIRMENT, AND PSYCHOMOTOR RETARDATION; CAVIPMR',
    'OMIM:616878': 'METABOLIC CRISES, RECURRENT, WITH RHABDOMYOLYSIS, CARDIAC ARRHYTHMIAS, AND NEURODEGENERATION; MECRCN',
    'OMIM:616892': 'NEPHROTIC SYNDROME, TYPE 12; NPHS12',
    'OMIM:616900': 'HYPOTONIA, INFANTILE, WITH PSYCHOMOTOR RETARDATION AND CHARACTERISTIC FACIES 3; IHPRF3',
    'OMIM:616954': 'YOU-HOOVER-FONG SYNDROME; YHFS',
    'OMIM:616973': 'MENTAL RETARDATION, AUTOSOMAL DOMINANT 42; MRD42',
    'OMIM:617011': 'MACROCEPHALY, DYSMORPHIC FACIES, AND PSYCHOMOTOR RETARDATION; MDFPMR',
    'OMIM:617013': 'HYPERMANGANESEMIA WITH DYSTONIA 2; HMNDYT2',
    'OMIM:617047': 'CARDIOMYOPATHY, FAMILIAL HYPERTROPHIC, 26; CMH26',
    'OMIM:617052': 'BONE MARROW FAILURE SYNDROME 3; BMFS3',
    'OMIM:617053': 'MIRAGE SYNDROME; MIRAGE',
    'OMIM:617093': 'GROWTH RETARDATION, IMPAIRED INTELLECTUAL DEVELOPMENT, HYPOTONIA, AND HEPATOPATHY; GRIDHH',
    'OMIM:617106': 'DEVELOPMENTAL AND EPILEPTIC ENCEPHALOPATHY 42; DEE42',
    'OMIM:617120': 'JOUBERT SYNDROME 27; JBTS27',
    'OMIM:617121': 'JOUBERT SYNDROME 28; JBTS28',
    'OMIM:617123': 'RETINITIS PIGMENTOSA 76; RP76',
    'OMIM:617137': 'FRONTOMETAPHYSEAL DYSPLASIA 2; FMD2',
    'OMIM:617146': 'ARTHROGRYPOSIS, DISTAL, WITH IMPAIRED PROPRIOCEPTION AND TOUCH; DAIPT',
    'OMIM:617183': 'HAREL-YOON SYNDROME; HAYOS',
    'OMIM:617193': 'ENCEPHALOPATHY, PROGRESSIVE, EARLY-ONSET, WITH BRAIN ATROPHY AND THIN CORPUS CALLOSUM; PEBAT',
    'OMIM:617194': 'Lethal congenital contracture syndrome 11',
    'OMIM:617222': 'SUDDEN CARDIAC FAILURE, INFANTILE; SCFI',
    'OMIM:617276': 'DEVELOPMENTAL AND EPILEPTIC ENCEPHALOPATHY 48; DEE48',
    'OMIM:617330': 'HYPOTONIA, ATAXIA, AND DELAYED DEVELOPMENT SYNDROME; HADDS',
    'OMIM:617350': 'DEVELOPMENTAL AND EPILEPTIC ENCEPHALOPATHY 52; DEE52',
    'OMIM:617397': 'PSEUDO-TORCH SYNDROME 2; PTORCH2',
    'OMIM:617433': 'RETINITIS PIGMENTOSA 78; RP78',
    'OMIM:617443': 'BLEEDING DISORDER, PLATELET-TYPE, 21; BDPLT21',
    'OMIM:617460': 'RETINITIS PIGMENTOSA 79; RP79',
    'OMIM:617481': 'NEURODEVELOPMENTAL DISORDER WITH MICROCEPHALY, HYPOTONIA, AND VARIABLE BRAIN ANOMALIES; NMIHBA',
    'OMIM:617537': 'RAHMAN SYNDROME; RMNS',
    'OMIM:617547': 'RETINAL DYSTROPHY WITH OR WITHOUT MACULAR STAPHYLOMA; RDMS',
    'OMIM:617622': 'JOUBERT SYNDROME 30; JBTS30',
    'OMIM:617675': 'MYOPATHY, MITOCHONDRIAL, AND ATAXIA; MMYAT',
    'OMIM:617681': 'BLEPHAROCHEILODONTIC SYNDROME 2; BCDS2',
    'OMIM:617694': 'AL KAISSI SYNDROME; ALKAS',
    'OMIM:617781': 'RETINITIS PIGMENTOSA 80; RP80',
    'OMIM:617805': 'RENAL HYPODYSPLASIA/APLASIA 3; RHDA3',
    'OMIM:617807': 'NEURODEVELOPMENTAL DISORDER WITH ATAXIC GAIT, ABSENT SPEECH, AND DECREASED CORTICAL WHITE MATTER; NDAGSCW',
    'OMIM:617829': 'EPILEPTIC ENCEPHALOPATHY, INFANTILE OR EARLY CHILDHOOD, 2; IECEE2',
    'OMIM:617873': 'COMBINED OXIDATIVE PHOSPHORYLATION DEFICIENCY 35; COXPD35',
    'OMIM:617948': 'ELLIPTOCYTOSIS 3; EL3',
    'OMIM:617954': 'MULTIPLE MITOCHONDRIAL DYSFUNCTIONS SYNDROME 6; MMDS6',
    'OMIM:618056': 'NEURODEVELOPMENTAL DISORDER WITH CEREBELLAR ATROPHY AND WITH OR WITHOUT SEIZURES; NEDCAS',
    'OMIM:618088': 'NEURODEVELOPMENTAL DISORDER WITH REGRESSION, ABNORMAL MOVEMENTS, LOSS OF SPEECH, AND SEIZURES; NEDAMSS',
    'OMIM:618129': 'MUSCULAR DYSTROPHY, LIMB-GIRDLE, AUTOSOMAL DOMINANT 4; LGMDD4',
    'OMIM:618138': 'MUSCULAR DYSTROPHY, LIMB-GIRDLE, AUTOSOMAL RECESSIVE 23; LGMDR23',
    'OMIM:618173': 'RETINITIS PIGMENTOSA 83; RP83',
    'OMIM:618174': 'CORTICAL DYSPLASIA, COMPLEX, WITH OTHER BRAIN MALFORMATIONS 9; CDCBM9',
    'OMIM:618225': 'MITOCHONDRIAL COMPLEX I DEFICIENCY, NUCLEAR TYPE 4; MC1DN4',
    'OMIM:618273': 'MEGA-CORPUS-CALLOSUM SYNDROME WITH CEREBELLAR HYPOPLASIA AND CORTICAL MALFORMATIONS; MCCCHCM',
    'OMIM:618316': 'INTELLECTUAL DEVELOPMENTAL DISORDER WITH CARDIAC DEFECTS AND DYSMORPHIC FACIES; IDDCDF',
    'OMIM:618325': 'LISSENCEPHALY 9 WITH COMPLEX BRAINSTEM MALFORMATION; LIS9',
    'OMIM:618404': 'LEUKODYSTROPHY, HYPOMYELINATING, 18; HLD18',
    'OMIM:618512': "O'DONNELL-LURIA-RODAN SYNDROME; ODLURO",
    'OMIM:618613': 'RETINITIS PIGMENTOSA 86; RP86',
    'OMIM:618826': 'RETINITIS PIGMENTOSA 88; RP88',
    'OMIM:PS123100': 'CRANIOSYNOSTOSIS 1; CRS1',
    'OMIM:PS173900': 'POLYCYSTIC KIDNEY DISEASE 1 WITH OR WITHOUT POLYCYSTIC LIVER DISEASE; PKD1',
    'OMIM:PS174050': 'POLYCYSTIC LIVER DISEASE 1 WITH OR WITHOUT KIDNEY CYSTS; PCLD1',
    'OMIM:PS252350': 'MOYAMOYA DISEASE 1; MYMY1',
    'OMIM:PS256100': 'NEPHRONOPHTHISIS 2; NPHP2',
    'OMIM:PS256300': 'NEPHROTIC SYNDROME, TYPE 2; NPHS2',
    'OMIM:PS258150': 'SPERMATOGENIC FAILURE 1; SPGF1',
    'OMIM:PS308230': 'IMMUNODEFICIENCY WITH HYPER-IgM, TYPE 3; HIGM3',
    'OMIM:PS610805': 'CONGENITAL ANOMALIES OF KIDNEY AND URINARY TRACT 2; CAKUT2',
    'ORPHA:71211': 'Neuromyelitis optica spectrum disorder',
    }

text_ontology = {v: k for k, v in ontology_text.items() }    

text_ontology['Schizophrenia cases and controls'] = 'DOID:5419'
text_ontology['CIRM iPSC Repository'] = 'UNKNOWN:UNKNOWN'
text_ontology['cardiovascular disease and risk factors'] = 'MONDO:0004995'
text_ontology['control'] = 'UNKNOWN:UNKNOWN'
text_ontology['epigenetic variation'] = 'UNKNOWN:UNKNOWN'
text_ontology['heart and blood vessel disease'] = 'MONDO:0004995'
text_ontology['hemorrhagic stroke'] = 'MONDO:0005098'
text_ontology['population'] = 'UNKNOWN:UNKNOWN'


# useful links:
#  https://docs.airr-community.org/en/latest/_sources/ontovoc/introduction_ontovoc.rst.txt
#  https://github.com/monarch-initiative/omim/blob/main/data/dipper/curie_map.yaml

#
# in preference order
#
disease_system = {
    "MONDO": 'https://data.bioontology.org/ontologies/MONDO_',
    "DOID": "http://purl.obolibrary.org/obo/DOID_",
    "HP": "http://purl.obolibrary.org/obo/HP_",
    "OMIM": "http://omim.org/entry/",
    "ORPHA": "http://www.orpha.net/ORDO/Orphanet_",
    "NCIT": "http://purl.obolibrary.org/obo/NCIT_",
    "UMLS": "http://linkedlifedata.com/resource/umls/id/",
    "MESH": "http://identifiers.org/mesh/",
}

#
# utilities to populate lookups
#

def omim_title(mimNumber):
    """Retrieve preferredTitle for OMIM."""
    mimNumber = mimNumber.split(':')[-1]
    headers = {
        "Accept": "application/json",
        "ApiKey": os.environ["OMIM_API_KEY"],
    }
    if mimNumber.startswith('PS'):
        response = requests.get(f"https://api.omim.org/api/entry/search?search=phenotypic_series_number:{mimNumber}", headers=headers)
        omim = response.json()
        assert 'omim' in omim, f"Response does not have 'omim' key. {omim}"
        assert 'searchResponse' in omim['omim'], f"Response does not have 'omim.searchResponse' key. {omim}"
        assert omim['omim']['searchResponse']['totalResults'] > 0, f"Response totalResults was 0. {omim}"
        assert 'entryList' in omim['omim']['searchResponse'], f"Response does not have 'omim.searchResponse.entryList' key. {omim}"
        entry = omim['omim']['searchResponse']['entryList'][0]['entry']
        assert 'titles' in entry, f"Response does not have 'omim.searchResponse.entryList[].titles' key. {omim}"
        assert 'preferredTitle' in entry['titles'], f"Response does not have 'omim.searchResponse.entryList[].titles.preferredTitle' key. {omim}"
        return entry['titles']['preferredTitle']
    else:
        response = requests.get(f"https://api.omim.org/api/entry?mimNumber={mimNumber}", headers=headers)
        omim = response.json()
        assert 'omim' in omim, f"Response does not have 'omim' key. {omim}"
        assert len(omim['omim']['entryList']) > 0, f"Response entryList length is 0. {omim}"
        return omim['omim']['entryList'][0]['entry']['titles']['preferredTitle']


def disease_ontology_xref(doid):
    """Retrieve all xrefs for a DOID:XXXX."""
    url = f"https://www.disease-ontology.org/api/metadata/{doid}"
    response = requests.get(url)
    disease = response.json()
    return disease.get('xrefs', None)

def omim_xref(mimNumber):
    """Retrieve all xrefs for a OMIM:XXXX."""
    mimNumber = mimNumber.split(':')[-1]
    headers = {
        "Accept": "application/json",
        "ApiKey": os.environ["OMIM_API_KEY"],
    }
    if mimNumber.startswith('PS'):
        response = requests.get(f"https://api.omim.org/api/entry/search?search=phenotypic_series_number:{mimNumber}&include=externalLinks", headers=headers)
        omim = response.json()
        assert 'omim' in omim, f"Response does not have 'omim' key. {omim}"
    else:
        response = requests.get(f"https://api.omim.org/api/entry?mimNumber={mimNumber}&include=externalLinks", headers=headers)
        omim = response.json()
        assert 'omim' in omim, f"Response does not have 'omim' key. {omim}"
    return omim

# disease_ontology_xrefs = {}
# for key in disease_text:
#     if 'DOID' not in key:
#         continue
#     xrefs = disease_ontology_xref(key)
#     if not xrefs:
#         continue
#     disease_ontology_xrefs[key] = xrefs
# {'DOID:0050589': ['EFO:0003767', 'KEGG:05321', 'MESH:D015212', 'NCI:C3138', 'OMIM:PS266600', 'SNOMEDCT_US_2021_03_01:155759008', 'UMLS_CUI:C0021390'], 'DOID:9744': ['EFO:0001359', 'GARD:10268', 'ICD10CM:E10', 'KEGG:04940', 'MESH:D003922', 'NCI:C2986', 'OMIM:222100', 'SNOMEDCT_US_2021_03_01:46635009', 'UMLS_CUI:C0011854'], 'DOID:1287': ['ICD9CM:429.2', 'MESH:D002318', 'NCI:C2931', 'SNOMEDCT_US_2021_03_01:266275004', 'UMLS_CUI:C0007222'], 'DOID:3393': ['EFO:0001645', 'ICD10CM:I20-I25', 'ICD10CM:I25', 'ICD10CM:I25.10', 'ICD9CM:410-414.99', 'ICD9CM:414.0', 'ICD9CM:414.9', 'MESH:D003324', 'MESH:D003327', 'MESH:D017202', 'NCI:C35505', 'NCI:C50625', 'OMIM:300464', 'OMIM:607339', 'OMIM:608316', 'OMIM:608318', 'OMIM:608320', 'OMIM:610947', 'OMIM:611139', 'OMIM:612030', 'OMIM:614293', 'SNOMEDCT_US_2021_03_01:194852007', 'SNOMEDCT_US_2021_03_01:233822007', 'SNOMEDCT_US_2021_03_01:41702007', 'SNOMEDCT_US_2021_03_01:53741008', 'UMLS_CUI:C0010054', 'UMLS_CUI:C0010068', 'UMLS_CUI:C0151744', 'UMLS_CUI:C0264694'], 'DOID:0060224': ['ICD9CM:427.31', 'MESH:D001281', 'NCI:C50466', 'SNOMEDCT_US_2021_03_01:266306001', 'UMLS_CUI:C0004238'], 'DOID:0014667': ['ICD10CM:E88.9', 'ICD9CM:277.9', 'MESH:D008659', 'NCI:C3235', 'SNOMEDCT_US_2021_03_01:75934005', 'UMLS_CUI:C0025517'], 'DOID:10652': ['EFO:0000249', 'GARD:10254', 'ICD10CM:G30', 'ICD9CM:331.0', 'KEGG:05010', 'MESH:D000544', 'NCI:C2866', 'SNOMEDCT_US_2021_03_01:73768007', 'UMLS_CUI:C0002395'], 'DOID:0060041': ['GARD:10248', 'MESH:D000067877'], 'DOID:2841': ['EFO:0000270', 'GARD:10246', 'ICD10CM:J45', 'ICD9CM:493', 'KEGG:05310', 'MESH:D001249', 'NCI:C28397', 'OMIM:600807', 'SNOMEDCT_US_2021_03_01:187687003', 'UMLS_CUI:C0004096'], 'DOID:5844': ['EFO:0000612', 'ICD10CM:I21', 'MESH:D009203', 'NCI:C27996', 'OMIM:608557', 'SNOMEDCT_US_2021_03_01:66514008', 'UMLS_CUI:C0027051'], 'DOID:1826': ['EFO:0000474', 'ICD10CM:G40.909', 'ICD9CM:345.9', 'MESH:D004827', 'NCI:C3020', 'SNOMEDCT_US_2021_03_01:267698007', 'UMLS_CUI:C0014544'], 'DOID:630': ['MESH:D030342', 'NCI:C3101', 'SNOMEDCT_US_2021_03_01:32895009', 'UMLS_CUI:C0019247'], 'DOID:5419': ['EFO:0000692', 'ICD10CM:F20', 'ICD9CM:295', 'MESH:D012559', 'NCI:C3362', 'OMIM:181500', 'SNOMEDCT_US_2021_03_01:58214004', 'UMLS_CUI:C0036341'], 'DOID:12377': ['GARD:7674', 'ICD10CM:G12.9', 'ICD9CM:335.1', 'MESH:D009134', 'NCI:C85075', 'SNOMEDCT_US_2021_03_01:5262007', 'UMLS_CUI:C0026847']}

#
# wget http://purl.obolibrary.org/obo/mondo/mondo-with-equivalents.json -O  /tmp/mondo-with-equivalents.json
# jq -rc '.graphs[] | .nodes[] | select(.meta.xrefs != null) |   [.id, .meta.definition.val, (.meta.xrefs[] | .val)]  '   /tmp/mondo-with-equivalents.json  > $OUTPUT_PATH/mondo-xrefs.json
#

#
# see anvil/util/bioontology_lookup.py
#



"""Client for reconciling terra subject."""

import logging
from anvil.gen3.drs_files import DRSFiles
from attrdict import AttrDict
from collections import defaultdict
import os


# drs entries
drs_files = None

# controls logging and drs lookup
sample_exceptions = []
SKIP_DRS = "skip_drs"


def _shorten_workspace(name):
    name = name.replace('AnVIL_', '')
    name = name.replace('CMG_', '')
    name = name.replace('CCDG_', '')
    return name


def _append_drs(sample):
    """Add ga4gh_drs_uri to blob."""
    global sample_exceptions
    if SKIP_DRS in sample_exceptions:
        return
    try:
        for key in sample.blobs.keys():
            filename = key.split('/')[-1]
            drs_file = drs_files.find_by_file_name(filename)
            assert drs_file, "Should have a gen3 sequencing record."
            sample.blobs[key]['ga4gh_drs_uri'] = drs_file['ga4gh_drs_uri']
            sample.blobs[key]['md5sum'] = drs_file['md5sum']
            assert sample.blobs[key]['ga4gh_drs_uri'], "Should have a ga4gh_drs_uri"
    except Exception as e:
        if sample.workspace_name not in sample_exceptions:
            logging.warn(f"DRS not found {sample.workspace_name} {sample.id} (supressing further errors for this workspace) {e}")
            sample_exceptions.append(sample.workspace_name)


class Sample(object):
    """Represent terra sample."""

    @staticmethod
    def skip_drs():
        """Disable DRS expensive lookup."""
        global sample_exceptions
        sample_exceptions.append(SKIP_DRS)

    def __init__(self, *args, workspace=None, blobs=None, sequencing=None, drs_file_path=None):
        """Pass all args to AttrDict."""
        global drs_files
        self._logger = logging.getLogger(__name__)
        self.attributes = AttrDict(*args)
        self.missing_blobs = True
        self.missing_sequence = False
        self.schema = workspace.sample_schema
        self.workspace_name = workspace.name
        self.missing_blob_path = False
        self.drs_file_path = drs_file_path
        self.blobs = self._find_blobs(blobs, sequencing)

        if not drs_files and drs_file_path:
            if os.path.isfile(drs_file_path):
                drs_files = DRSFiles(drs_file_path)
            else:
                if 'drs_file_path' not in sample_exceptions:
                    self._logger.warn(f"{drs_file_path} should exist. Please export anvil_extract drs-file")
                    sample_exceptions.append('drs_file_path')
        if drs_files:
            _append_drs(self)

    def _find_blobs(self, blobs, sequencing):
        """Find all blobs associated with sample."""
        self._logger.debug("_find_blobs")
        blob_names = [(property_name, blob_name) for property_name, blob_name in self.attributes.attributes.items() if isinstance(blob_name, str) and blob_name.startswith('gs://')]
        my_blobs = []
        for property_name, blob_name in blob_names:
            if 'md5' in blob_name:
                # print(f'skipping md5 {blob_name}')
                continue
            blob = blobs.get(blob_name, None)
            if blob:
                blob['property_name'] = property_name
            my_blobs.append(blob)
        self.missing_blobs = len([b for b in my_blobs if b]) == 0
        return {b['name']: b for b in my_blobs if b}

    def __repr__(self):
        """Return attributes."""
        return str(self.attributes)

    @property
    def blob_sizes(self):
        """Aggregate blob sizes by property name."""
        _blob_sizes = defaultdict(int)
        for b in self.blobs.values():
            _blob_sizes[b['property_name']] += b['size']
        return _blob_sizes

    @property
    def id(self):
        """Delegate to subclass."""
        raise Exception('Not implemented')

    @property
    def subject_id(self):
        """Delegate to subclass."""
        raise Exception('Not implemented')

    @property
    def inconsistent_entityName(self):
        """Delegate to subclass."""
        raise Exception('Not implemented')

    @property
    def misspelled_subject(self):
        """Delegate to subclass."""
        raise Exception('Not implemented')


class CCDGSample(Sample):
    """Extend Sample class."""

    def __init__(self, *args, workspace=None, blobs=None, sequencing=None, drs_file_path=None):
        """Call super."""
        super().__init__(*args, workspace=workspace, blobs=blobs, sequencing=sequencing, drs_file_path=drs_file_path)

    @property
    def id(self):
        """Deduce id."""
        if 'project' not in self.attributes.attributes:
            # return f"{_shorten_workspace(self.workspace_name)}/Sa/{self.attributes.name}"
            return self.attributes.name
        # format the gen3 uses
        if 'collaborator_sample_id' not in self.attributes.attributes:
            # return f"{self.attributes.attributes['project']}_{self.attributes.attributes['sample']}"
            return self.attributes.attributes['sample']
        # return f"{self.attributes.attributes['project']}_{self.attributes.attributes['collaborator_sample_id']}"
        return self.attributes.attributes['collaborator_sample_id']

    @property
    def subject_id(self):
        """Deduce id."""
        if self.inconsistent_entityName:
            if self.inconsistent_subject:
                # this project has no subjects?
                if self.workspace_name == 'AnVIL_CCDG_WashU_CVD_EOCAD_BioImage_WGS':
                    return None
                if 'participent' not in self.attributes.attributes:
                    return None
                return self.attributes.attributes.participent
            return self.attributes.attributes.participant
        return self.attributes.attributes.participant.entityName

    @property
    def inconsistent_entityName(self):
        """Flag entityName inconsistencies."""
        if self.inconsistent_subject or 'entityName' not in self.attributes.attributes.participant:
            return True
        return False

    @property
    def inconsistent_subject(self):
        """Flag misspellings."""
        if 'participant' not in self.attributes.attributes:
            return True
        return False


class CMGSample(Sample):
    """Extend Sample class."""

    def __init__(self, *args, workspace=None, blobs=None, sequencing=None, drs_file_path=None):
        """Call super."""
        super().__init__(*args, workspace=workspace, blobs=blobs, sequencing=sequencing, drs_file_path=drs_file_path)

    def _find_blobs(self, blobs, sequencing):
        """Find all blobs associated with sample."""
        blob_names = [(property_name, blob_name) for property_name, blob_name in self.attributes.attributes.items() if isinstance(blob_name, str) and blob_name.startswith('gs://')]
        if sequencing:
            try:
                sequence = [s for s in sequencing if 'collaborator_sample_id' in s['attributes'] and s['attributes']['collaborator_sample_id'] == self.id]
            except Exception as e:
                print(f'collaborator_sample_id not present {e}')
                print(self.id)
                print(sequencing[0]['attributes'].keys())
                print(sequencing[0])
                sequence = []

            if len(sequence) == 0:
                try:
                    sequence = [s for s in sequencing if 'sample_alias' in s['attributes'] and s['attributes']['sample_alias'] == self.id]
                except Exception:
                    print('sample_alias not present')
                    print(self.id)
                    print(sequencing[0]['attributes'].keys())
                    print(sequencing[0])
                    sequence = []

            if len(sequence):
                sequence = sequence[0]['attributes']
                blob_names.extend([(property_name, blob_name) for property_name, blob_name in sequence.items() if isinstance(blob_name, str) and blob_name.startswith('gs://')])
            else:
                self.missing_sequence = True

        # assert len(blob_names) > 0, self.workspace_name
        my_blobs = []
        for property_name, blob_name in blob_names:
            blob = blobs.get(blob_name, None)
            if blob:
                blob['property_name'] = property_name
            my_blobs.append(blob)
        # assert len(my_blobs) > 0, self.workspace_name
        self.missing_blobs = not len([b for b in my_blobs if not b]) == 0
        return AttrDict({b['name']: b for b in my_blobs if b})

    @property
    def subject_id(self):
        """Deduce id."""
        if self.inconsistent_subject:
            if 'participant' in self.attributes.attributes:
                return self.attributes.attributes.participant.entityName
            if 'subject_id' in self.attributes.attributes:
                return self.attributes.attributes.subject_id
            if 'participant_id' in self.attributes.attributes:
                return self.attributes.attributes.participant_id
            print(self.attributes)
            print(self.attributes.keys())
            assert False, 'inconsistent_subject unknown subject identifier'
        if '01-subject_id' not in self.attributes.attributes:
            print(self.attributes)
            print(self.attributes.keys())
        return self.attributes.attributes['01-subject_id']

    @property
    def inconsistent_subject(self):
        """Flag subject id inconsistencies."""
        if '01-subject_id' in self.attributes.attributes:
            return False
        return True

    @property
    def inconsistent_entityName(self):
        """Flag entityName inconsistencies."""
        return False

    @property
    def id(self):
        """Deduce id."""
        # return f"{_shorten_workspace(self.workspace_name)}/Sa/{self.attributes.name}"
        return self.attributes['name']


class GTExSample(Sample):
    """Extend Sample class."""

    def __init__(self, *args, workspace=None, blobs=None, sequencing=None, drs_file_path=None):
        """Call super."""
        super().__init__(*args, workspace=workspace, blobs=blobs, sequencing=sequencing, drs_file_path=drs_file_path)

    @property
    def id(self):
        """Deduce id."""
        # return f"{_shorten_workspace(self.workspace_name)}/Sa/{self.attributes.name}"
        return self.attributes.name

    @property
    def subject_id(self):
        """Deduce id."""
        return self.attributes.attributes.participant.entityName

    @property
    def inconsistent_entityName(self):
        """Flag entityName inconsistencies."""
        return False

    @property
    def misspelled_subject(self):
        """Flag misspellings."""
        return False

    @property
    def inconsistent_subject(self):
        """Flag misspellings."""
        return False


class ThousandGenomesSample(Sample):
    """Extend Sample class."""

    def __init__(self, *args, workspace=None, blobs=None, sequencing=None, drs_file_path=None):
        """Call super."""
        super().__init__(*args, workspace=workspace, blobs=blobs, sequencing=sequencing, drs_file_path=drs_file_path)

    @property
    def id(self):
        """Deduce id."""
        # return f"{self.workspace_name}/Sa/{self.attributes.name}"
        return self.attributes.name

    @property
    def subject_id(self):
        """Deduce id."""
        return self.attributes.attributes.participant

    @property
    def inconsistent_entityName(self):
        """Flag entityName inconsistencies."""
        return False

    @property
    def misspelled_subject(self):
        """Flag misspellings."""
        return False

    @property
    def inconsistent_subject(self):
        """Flag misspellings."""
        return False


class eMERGESample(Sample):
    """Extend Sample class."""

    def __init__(self, *args, workspace=None, blobs=None, sequencing=None, drs_file_path=None):
        """Call super."""
        super().__init__(*args, workspace=workspace, blobs=blobs, sequencing=sequencing, drs_file_path=drs_file_path)

    @property
    def id(self):
        """Deduce id."""
        return self.attributes.name

    @property
    def subject_id(self):
        """Deduce id."""
        return self.attributes.attributes.participant.entityName

    @property
    def inconsistent_entityName(self):
        """Flag entityName inconsistencies."""
        return False

    @property
    def misspelled_subject(self):
        """Flag misspellings."""
        return False

    @property
    def inconsistent_subject(self):
        """Flag misspellings."""
        return False


class NHGRISample(Sample):
    """Extend Sample class."""

    def __init__(self, *args, workspace=None, blobs=None, sequencing=None, drs_file_path=None):
        """Call super."""
        super().__init__(*args, workspace=workspace, blobs=blobs, sequencing=sequencing, drs_file_path=drs_file_path)

    @property
    def id(self):
        """Deduce id."""
        return self.attributes.name

    @property
    def subject_id(self):
        """Deduce id."""
        return self.attributes.attributes.participant.entityName

    @property
    def inconsistent_entityName(self):
        """Flag entityName inconsistencies."""
        return False

    @property
    def misspelled_subject(self):
        """Flag misspellings."""
        return False

    @property
    def inconsistent_subject(self):
        """Flag misspellings."""
        return False


class NIMHSample(Sample):
    """Extend Sample class."""

    def __init__(self, *args, workspace=None, blobs=None, sequencing=None, drs_file_path=None):
        """Call super."""
        super().__init__(*args, workspace=workspace, blobs=blobs, sequencing=sequencing, drs_file_path=drs_file_path)

    @property
    def id(self):
        """Deduce id."""
        return self.attributes.name

    @property
    def subject_id(self):
        """Deduce id."""
        try:
            return self.attributes.attributes.participant.entityName
        except Exception:
            return self.attributes.attributes.subject_id

    @property
    def inconsistent_entityName(self):
        """Flag entityName inconsistencies."""
        return 'participant' not in self.attributes.attributes

    @property
    def misspelled_subject(self):
        """Flag misspellings."""
        return False

    @property
    def inconsistent_subject(self):
        """Flag misspellings."""
        return False


class PAGESample(Sample):
    """Extend Sample class."""

    def __init__(self, *args, workspace=None, blobs=None, sequencing=None, drs_file_path=None):
        """Call super."""
        super().__init__(*args, workspace=workspace, blobs=blobs, sequencing=sequencing, drs_file_path=drs_file_path)

    @property
    def id(self):
        """Deduce id."""
        return self.attributes.name

    @property
    def subject_id(self):
        """Deduce id."""
        try:
            return self.attributes.attributes.participant_id
        except Exception:
            pass
        try:
            return self.attributes.attributes.subject_id
        except Exception:
            pass
        raise Exception(f"can't find participant_id or subject_id  {self.attributes}")

    @property
    def inconsistent_entityName(self):
        """Flag entityName inconsistencies."""
        return 'participant' not in self.attributes.attributes

    @property
    def misspelled_subject(self):
        """Flag misspellings."""
        return False

    @property
    def inconsistent_subject(self):
        """Flag misspellings."""
        return False


def sample_factory(*args, **kwargs):
    """Return a specialized Subject class instance."""
    if 'CCDG' in kwargs['workspace'].name.upper():
        return CCDGSample(*args, **kwargs)
    if 'CMG' in kwargs['workspace'].name.upper():
        return CMGSample(*args, **kwargs)
    if 'GTEX' in kwargs['workspace'].name.upper():
        return GTExSample(*args, **kwargs)
    if '1000G-HIGH-COVERAGE' in kwargs['workspace'].name.upper():
        return ThousandGenomesSample(*args, **kwargs)
    if 'ANVIL_EMERGE' in kwargs['workspace'].name.upper():
        return eMERGESample(*args, **kwargs)
    if 'NHGRI' in kwargs['workspace'].name.upper():
        return NHGRISample(*args, **kwargs)
    if 'NIMH' in kwargs['workspace'].name.upper():
        return NIMHSample(*args, **kwargs)
    if 'PAGE' in kwargs['workspace'].name.upper():
        return PAGESample(*args, **kwargs)
    raise Exception(f'Not implemented {kwargs["workspace"].name.upper()}')



import json
import os

import click
import logging

import requests

from anvil.etl.loaders.fhir import _extract_workspace_mapping, _log_console_link
from anvil.etl.loaders.fhir.implementation_guide import _enable_ig, _get_ig
from anvil.etl.transformers.fhir_writer import ensure_data_store_name
from anvil.etl.utilities import chunker
from anvil.etl.utilities.shell_helper import run_cmd

logger = logging.getLogger(__name__)


@click.group(name='data-store')
@click.pass_context
def data_store_cli(ctx):
    pass  # dummy


@data_store_cli.command(name='studies')
@click.option('--consortium', default=None, help='Filter, only this consortium.')
@click.option('--workspace', default=None, help='Filter, only this workspace.')
@click.pass_context
def _research_studies(ctx, consortium, workspace):
    """Print transformed ResearchStudy data-store info to stdout."""
    workspace_mapping = _extract_workspace_mapping(ctx.obj['output_path'])
    print(json.dumps(workspace_mapping['data_store']))


@data_store_cli.command(name='create')
@click.pass_context
def create_data_stores(ctx, token=None):
    """Create data_stores, enable IG."""
    # TODO - filter consortium, workspace, data-store
    output_path = ctx.obj["output_path"]
    workspace_mapping = _extract_workspace_mapping(output_path)
    list_cmd = f'gcloud beta healthcare fhir-stores list --dataset=$GOOGLE_DATASET  --location=$GOOGLE_LOCATION  --format="table[no-heading](ID)"'
    data_store_list = run_cmd(list_cmd)

    cmds = [f"gcloud beta healthcare fhir-stores create {data_store} --dataset=$GOOGLE_DATASET --location=$GOOGLE_LOCATION --version=R4 --enable-update-create" for data_store in workspace_mapping['data_store'].keys() if data_store not in data_store_list]
    if len(cmds) > 0:
        run_cmd("\n".join(cmds))
    data_store_list = run_cmd(list_cmd)
    error = False
    for data_store in workspace_mapping['data_store']:
        if data_store not in data_store_list:
            logger.error(f"{data_store} not found in:\n{data_store_list}")
            error = True

    if not error:
        logger.info("All data stores exist. Loading IG...")
    cmds = [f"gcloud beta healthcare fhir-stores import gcs  {data_store}  --dataset=$GOOGLE_DATASET  --location=$GOOGLE_LOCATION --gcs-uri=gs://$GOOGLE_BUCKET/fhir/IG/*.json --content-structure=resource-pretty --async" for data_store in workspace_mapping['data_store'].keys() if data_store in data_store_list]
    if len(cmds) > 0:
        cmd_results = run_cmd("\n".join(cmds))
        while True:
            logger.info(f"Waiting for import to complete...")
            check_operations = json.loads(run_cmd('sleep 5 ; gcloud beta healthcare operations list --dataset $GOOGLE_DATASET --location $GOOGLE_LOCATION   --format \'json(name)\'   --filter="done:false"'))
            if len(check_operations) == 0:
                break

    logger.info("IG loaded on all. Enabling IG...")
    if not token:
        token = run_cmd("gcloud auth application-default print-access-token")
    for data_store in workspace_mapping['data_store'].keys():
        if data_store in data_store_list:
            _enable_ig(ctx, data_store, token)


@data_store_cli.command(name='upload')
@click.option('--consortium', default=None, help='Filter, only this consortium.', show_default=True)
@click.option('--workspace', default=None, help='Filter, only this workspace', show_default=True)
@click.option('--drop', default=False, help='Delete FHIR daa in bucket first', show_default=True, is_flag=True)
@click.option('--check', default=False, help='Delete FHIR daa in bucket first', show_default=True, is_flag=True)
@click.option('--go_fast', default=True, help='Copy entire fhir/ sdir in one go.', show_default=True, is_flag=True)
@click.pass_context
def upload(ctx, consortium, workspace, drop, check, go_fast):
    """Copy FHIR resources from local disk to bucket."""
    output_path = ctx.obj["output_path"]
    workspace_consortium_mapping = _extract_workspace_mapping(output_path)['consortium']
    uploaded_workspaces = []
    for consortium_name, workspaces in workspace_consortium_mapping.items():
        for mapping in workspaces:
            if consortium and consortium != consortium_name:
                continue
            if workspace and workspace != mapping['name']:
                continue
            data_store = mapping['data_store']
            dir_name = f"{output_path}/fhir/{data_store}/{consortium_name}/{mapping['name']}"
            assert os.path.isdir(dir_name), f"{dir_name} does not exist?"
            uploaded_workspaces.append(f"fhir/{data_store}/{consortium_name}/{mapping['name']}")

    # for group in chunker(copy_commands, 20):
    #     run_cmd('\n'.join(group))
    # for group in chunker(check_commands, 20):
    #     results = run_cmd('\n'.join(group))

    if drop:
        for dir_name in uploaded_workspaces:
            run_cmd(f"gsutil rm -r gs://$GOOGLE_BUCKET/{dir_name}")

    if go_fast:
        run_cmd(f"gsutil -m cp -J -r {output_path}/fhir/ gs://$GOOGLE_BUCKET")
    else:
        for group in chunker(uploaded_workspaces, 20):
            cmds = [f"gsutil -m cp -J -r {output_path}/{dir_name}/ gs://$GOOGLE_BUCKET" for dir_name in group]
            run_cmd('\n'.join(cmds))

    if check:
        results = run_cmd(f"gsutil ls -r gs://$GOOGLE_BUCKET/fhir")
        for dir_name in uploaded_workspaces:
            assert dir_name in results, ('missing.obj', dir_name, results)


@data_store_cli.command(name='load')
@click.pass_context
def load_data_stores(ctx):
    """Load data from bucket into data stores."""
    output_path = ctx.obj["output_path"]
    workplace_mapping = _extract_workspace_mapping(output_path)
    workspace_datastore_mapping = workplace_mapping['data_store']

    cmds = []
    for data_store, workspaces in workspace_datastore_mapping.items():
        for mapping in workspaces:
            object_path = f"fhir/{mapping['data_store']}/{mapping['consortium']}/{mapping['name']}"
            cmds.append(
                f"gcloud healthcare fhir-stores import gcs {data_store} --location=$GOOGLE_LOCATION --dataset=$GOOGLE_DATASET --content-structure=resource --async --gcs-uri=gs://$GOOGLE_BUCKET/{object_path}/**.ndjson"
            )
            # for subdir in ['public', 'protected']:
            #     cmds.append(f"gcloud healthcare fhir-stores import gcs {data_store} --location=$GOOGLE_LOCATION --dataset=$GOOGLE_DATASET --content-structure=resource --async --gcs-uri=gs://$GOOGLE_BUCKET/{object_path}/{subdir}/*.ndjson"
    for group in chunker(cmds, 20):
        run_cmd('\n'.join(group))
    _log_console_link(logger)


@data_store_cli.command(name='delete')
@click.pass_context
def delete(ctx):
    """Delete data stores."""
    # TODO - filter consortium, workspace, data-store
    output_path = ctx.obj["output_path"]
    workplace_mapping = _extract_workspace_mapping(output_path)
    list_cmd = f'gcloud beta healthcare fhir-stores list --dataset=$GOOGLE_DATASET  --location=$GOOGLE_LOCATION  --format="table[no-heading](ID)"'
    data_store_list = run_cmd(list_cmd)
    cmds = [f"gcloud beta healthcare fhir-stores delete {data_store} --dataset=$GOOGLE_DATASET --location=$GOOGLE_LOCATION --quiet" for data_store in workplace_mapping['data_store'].keys() if data_store in data_store_list]
    if len(cmds) > 0:
        run_cmd("\n".join(cmds))
    data_store_list = run_cmd(list_cmd)
    for data_store in workplace_mapping['data_store']:
        if data_store in data_store_list:
            logger.error(f"{data_store} delete failed, found in:\n{data_store_list}")
    logger.info("All data stores deleted.")


@data_store_cli.command(name='ls')
@click.option('--consortium', default=None, help='Filter, only this consortium.')
@click.option('--workspace', default=None, help='Filter, only this workspace.')
@click.option('--token',
              default=None,
              help='gcloud access token. if null cmd will exec `gcloud auth application-default print-access-token`',
              show_default=True)
@click.pass_context
def _data_stores(ctx, consortium, workspace, token):
    """Print server data-store info to stdout."""
    workspace_mapping = _extract_workspace_mapping(ctx.obj['output_path'])

    if not token:
        token = run_cmd("gcloud auth application-default print-access-token")

    list_cmd = f'gcloud beta healthcare fhir-stores list --dataset=$GOOGLE_DATASET  --location=$GOOGLE_LOCATION  --format="table[no-heading](ID)"'
    data_store_list = run_cmd(list_cmd)

    for data_store in data_store_list.split('\n'):
        print(json.dumps(_get_ig(ctx, data_store, token)))




"""Query sqlite for gen3 sequencing records, for more see bin/anvil_extract."""

import sqlite3


def dict_factory(cursor, row):
    """Return results as a dict."""
    d = {}
    for idx, col in enumerate(cursor.description):
        d[col[0]] = row[idx]
    return d


class DRSFiles:
    """Lookup from gen3 sequencing."""

    def __init__(self, sqlite_path):
        """Lookup from gen3 sequencing."""
        self._conn = sqlite3.connect(sqlite_path)
        self._conn.row_factory = dict_factory
        self._cur = self._conn.cursor()

    def find_by_file_name(self, file_name):
        """Find using file_name."""
        return self._cur.execute("SELECT * FROM drs_file where file_name =?", (file_name, )).fetchone()

    def find_by_md5sum(self, md5sum):
        """Find using md5sum."""
        return self._cur.execute("SELECT * FROM drs_file where md5sum =?", (md5sum, )).fetchone()



from collections import defaultdict
import json
import os
import uuid
import logging
import re
import itertools

import fhirclient.models.patient as FHIRPatient
import fhirclient.models.organization as FHIROrganization
import fhirclient.models.researchstudy as FHIRResearchStudy
import fhirclient.models.researchsubject as FHIRResearchSubject
import fhirclient.models.specimen as FHIRSpecimen
import fhirclient.models.task as FHIRTask
import fhirclient.models.documentreference as FHIRDocumentReference
import fhirclient.models.observation as FHIRObservation
import fhirclient.models.fhirreference as FHIRReference
import fhirclient.models.identifier as FHIRIdentifier
import fhirclient.models.practitioner as FHIRPractitioner
import fhirclient.models.practitionerrole as FHIRPractitionerRole

from anvil.etl.utilities.disease_normalizer import ontology_text, disease_system, text_ontology

logger = logging.getLogger(__name__)

# note coordinate with StructureDefintions /fhir/config.yaml::canonical
CANONICAL = "https://nih-ncpi.github.io/ncpi-fhir-ig"


def _ref(fhir_resource):
    """Create a local reference for a fhir resource."""
    return FHIRReference.FHIRReference({'reference': f"{fhir_resource.resource_type}/{fhir_resource.id}"})


def _id(*args):
    """Create legal fhir id, a reproducible SHA-1 hash of a workspace_name and resource_id."""
    name = '-'.join([x for x in args])
    namespace = bytes(bytearray("pyAnVIL".ljust(16), 'utf-8'))
    namespace_uuid = uuid.UUID(bytes=namespace)
    return str(uuid.uuid5(namespace_uuid, name))


def _identifier(workspace, terra_entity):
    """Create FHIR compliant identifier from terra entity"""
    js = {
        "system": f"https://anvil.terra.bio/#workspaces/anvil-datastorage/{workspace.workspace.name}",
        "value": f"{terra_entity['entityType']}/{terra_entity['name']}",
    }
    return FHIRIdentifier.Identifier(js)


def _document_reference_identifier(workspace, output_source, output_property, specimen):
    """Create FHIR compliant identifier for document reference, add the blob origination entity and property."""
    js = {
        "system": f"https://anvil.terra.bio/#workspaces/anvil-datastorage/{workspace.workspace.name}",
        "value": f"{specimen['entityType']}/{specimen['name']}#{output_source}/{output_property}",
    }
    return FHIRIdentifier.Identifier(js)


def _family_identifier(workspace, family_id):
    """Create FHIR compliant identifier for family_id."""
    js = {
        "system": f"https://anvil.terra.bio/#workspaces/anvil-datastorage/{workspace.workspace.name}#family_id",
        "value": family_id
    }
    return FHIRIdentifier.Identifier(js)


def _patient_identifier(workspace, id, label):
    """Create FHIR compliant identifier for patient id."""
    js = {
        "system": f"https://anvil.terra.bio/#workspaces/anvil-datastorage/{workspace.workspace.name}#{label}",
        "value": id
    }
    return FHIRIdentifier.Identifier(js)


def _fhir_id(*args):
    """Create legal fhir id."""
    return re.sub(r'[^A-Za-z0-9\-\.]', "-", ".".join(str(a) for a in args))[-64:]


def ensure_data_store_name(workspace):
    """Create data store name from phsId and dataUseRestriction."""
    data_store_name = 'pending'
    if workspace.consortium_name == 'Public':
        data_store_name = 'public'
    if 'tracker' in workspace and workspace['tracker'] and 'phsId' in workspace['tracker'] and workspace['tracker']['phsId']:
        data_store_name = workspace['tracker']['phsId']
        if not data_store_name or not data_store_name.startswith('phs'):
            data_store_name = 'pending'
        else:
            if "library:dataUseRestriction" in workspace['tracker'] and 'tbd' != workspace['tracker']['library:dataUseRestriction'].lower():
                data_store_name = f"{data_store_name}-{workspace['tracker']['library:dataUseRestriction']}"
    return data_store_name.replace(' ', '_')


def _create_administrative_entities(workspace, consortium_name):
    """Generate FHIR Organization, ResearchStudy, etc."""
    workspace_name = workspace.workspace.name

    anvil_org = FHIROrganization.Organization({'id': 'anvil'})
    anvil_org.identifier = [
        FHIRIdentifier.Identifier({
            "system": "https://anvil.terra.bio/program",
            "value": 'anvil',
        })
    ]

    consortium_org = FHIROrganization.Organization({'id': _fhir_id(consortium_name)})
    consortium_org.partOf = _ref(anvil_org)
    consortium_org.identifier = [
        FHIRIdentifier.Identifier({
            "system": "https://anvil.terra.bio/consortium",
            "value": consortium_name,
        })
    ]

    data_store_name = ensure_data_store_name(workspace)    
    data_store_org = FHIROrganization.Organization({'id': _fhir_id(data_store_name)})
    data_store_org.partOf = _ref(consortium_org)
    data_store_org.identifier = [
        FHIRIdentifier.Identifier({
            "system": "https://anvil.terra.bio/#FHIR/data-store",
            "value": data_store_name,
        })
    ]

    practitioner = FHIRPractitioner.Practitioner({'id': _fhir_id(workspace.practitioner)})
    workspace_org = FHIROrganization.Organization({'id': _id('Organization', workspace_name)})
    workspace_org.partOf = _ref(data_store_org)
    workspace_org.identifier = [FHIRIdentifier.Identifier({
        "system": "https://anvil.terra.bio/#workspaces/anvil-datastorage/",
        "value": f"{workspace_name}",
    })]
    practitioner_role = FHIRPractitionerRole.PractitionerRole(
        {'id': _id('PractitionerRole', workspace_name, workspace.practitioner)}
    )
    practitioner_role.practitioner = _ref(practitioner)
    practitioner_role.organization = _ref(workspace_org)
    practitioner_role.identifier = [FHIRIdentifier.Identifier({
        "system": "https://anvil.terra.bio/#workspaces/anvil-datastorage/",
        "value": f"{workspace_name}",
    })]

    research_study = FHIRResearchStudy.ResearchStudy({'id': _fhir_id(workspace_name), 'status': 'completed', })
    research_study.sponsor = _ref(workspace_org)
    research_study.identifier = [
        FHIRIdentifier.Identifier({
            "system": "https://anvil.terra.bio/#workspaces/anvil-datastorage/",
            "value": f"{workspace_name}",
        }),
        FHIRIdentifier.Identifier({
            "system": "https://anvil.terra.bio/#FHIR/data-store",
            "value": f"{data_store_name}",
        }),
        FHIRIdentifier.Identifier({
            "system": "https://anvil.terra.bio/#consortium",
            "value": consortium_name,
        }),
    ]

    return (
        anvil_org,
        consortium_org,
        practitioner,
        data_store_org,
        workspace_org,
        practitioner_role,
        research_study
    )


def _make_fhir_task_NEW(fhir_patient, task, workspace, workspace_name):
    """Generate Task, etc."""
    for output_source, output in task['outputs'].items():
        for output_property, blob in output.items():
            _task_id_keys.append(blob['url'])
    task_id = _id(*_task_id_keys)
    identifiers = [_identifier(workspace, task)]
    inputs = []
    for _input in task['inputs']:
        assert 'fhir_entity' in _input, 'missing.fhir_entity.in.task'
        inputs.append(
            {
                'type': {'coding': [{'code': 'Reference'}]},
                'valueReference': {
                    'reference': f"{_input['fhir_entity']}/{_id(workspace_name, _input['fhir_entity'], _input['name'])}"}
            }
        )

    task = {
      "id": task_id,
      "identifier": identifiers,
      "input": inputs,
      "intent": "unknown",
      "output": [
        {
          "type": {
            "coding": [
              {
                "code": "Reference"
              }
            ]
          },
          "valueReference": {
            "reference": "DocumentReference/c07e3b64-f677-5ef7-9bb0-8a2a188592c2"
          }
        },
        {
          "type": {
            "coding": [
              {
                "code": "Reference"
              }
            ]
          },
          "valueReference": {
            "reference": "DocumentReference/f21602c9-f5ad-5863-9fa3-cef957311a5f"
          }
        },
        {
          "type": {
            "coding": [
              {
                "code": "Reference"
              }
            ]
          },
          "valueReference": {
            "reference": "DocumentReference/19162a3a-346a-54de-8587-a4aa5d7b1a5b"
          }
        },
        {
          "type": {
            "coding": [
              {
                "code": "Reference"
              }
            ]
          },
          "valueReference": {
            "reference": "DocumentReference/c07e3b64-f677-5ef7-9bb0-8a2a188592c2"
          }
        },
        {
          "type": {
            "coding": [
              {
                "code": "Reference"
              }
            ]
          },
          "valueReference": {
            "reference": "DocumentReference/f21602c9-f5ad-5863-9fa3-cef957311a5f"
          }
        }
      ],
      "status": "accepted",
      "resourceType": "Task"
    }


def _generate_specimen_descendants(workspace, patient, fhir_patient, details):
    """Generate FHIR Specimen, Task and DocumentReference."""
    workspace_name = workspace.workspace.name

    if 'specimens' in patient:
        for specimen in patient['specimens']:
            fhir_specimen = FHIRSpecimen.Specimen({'id': _id(workspace_name, 'Specimen', specimen['name']), 'subject': _ref(fhir_patient).as_json()})
            fhir_specimen.identifier = [_identifier(workspace, specimen)]
            yield fhir_specimen
            if details:
                yield _terra_observation(workspace, specimen, fhir_specimen)
            if 'tasks' in specimen:
                for task in specimen['tasks']:
                    fhir_task = yield from _make_fhir_task(fhir_patient, task, workspace, workspace_name)
                    yield fhir_task
                    if details:
                        yield _terra_observation(workspace, task, fhir_task)


def _make_fhir_task(fhir_patient, task, workspace, workspace_name):
    """Create task and its descendants."""
    # create unique task id
    _task_id_keys = [input['name'] for input in task['inputs']]
    for output_source, output in task['outputs'].items():
        for output_property, blob in output.items():
            _task_id_keys.append(blob['url'])
    task_id = _id(*_task_id_keys)
    fhir_task = FHIRTask.Task({'id': task_id, 'input': [], 'output': [], 'status': 'accepted', 'intent': 'unknown'})
    fhir_task.identifier = [_identifier(workspace, task)]
    for input in task['inputs']:
        assert 'fhir_entity' in input, 'missing.fhir_entity.in.task'
        fhir_task.input.append(FHIRTask.TaskInput(
            {
                'type': {'coding': [{'code': 'Reference'}]},
                'valueReference': {
                    'reference': f"{input['fhir_entity']}/{_id(workspace_name, input['fhir_entity'], input['name'])}"}
            }
        ))
    seen_already = set()
    for output_source, output in task['outputs'].items():
        for output_property, blob in output.items():
            assert 'url' in blob, ('missing.url.in.blob', output_source, output_property)
            document_reference = FHIRDocumentReference.DocumentReference(
                {
                    'id': _id(blob['url']),
                    'status': 'current',
                    'content': [{'attachment': {'url': blob['url']}}],
                    'subject': _ref(fhir_patient).as_json()
                }
            )
            document_reference.identifier = [
                _document_reference_identifier(workspace, output_source, output_property, task['inputs'][0])]

            # FHIRDocumentReference.DocumentReferenceContent({'attachment': {'url': blob['url']} })
            fhir_task.output.append(
                FHIRTask.TaskOutput(
                    {'type': {'coding': [{'code': 'Reference'}]}, 'valueReference': _ref(document_reference).as_json()}
                )
            )
            # multiple tasks can refer to same document, so de-duplicate
            if document_reference.id not in seen_already:
                yield document_reference
            seen_already.add(document_reference.id)
    yield fhir_task
    return fhir_task


def _create_individual(workspace, patient, workspace_org, research_study):
    """Create FHIR resources from Patient, ResearchSubject."""
    workspace_name = workspace.workspace.name

    fhir_patient = FHIRPatient.Patient(
        {
            'id': _id(workspace_name, 'Patient', patient['name']),
            'managingOrganization': _ref(workspace_org).as_json()
        }
    )
    if 'gender' in patient:
        fhir_patient.gender = patient['gender']
    fhir_patient.identifier = [_identifier(workspace, patient)]

    research_subject = FHIRResearchSubject.ResearchSubject(
        {
            'id': _id(workspace_name, 'ResearchSubject', patient['name']),
            'status': 'on-study',
            'individual': _ref(fhir_patient).as_json(),
            'study': _ref(research_study).as_json()
        }
    )
    research_subject.identifier = [_identifier(workspace, patient)]
    return fhir_patient, research_subject


def _disease_coding(disease):
    """Given a term, lookup the disease ontology match."""
    curie = text_ontology.get(disease, None)
    assert curie, f"could not find curie for {disease}"
    text = ontology_text.get(curie)
    prefix, code = curie.split(':')
    system = disease_system.get(prefix)
    return {
        "system": system,
        "code": code,
        "display": text
    }


def _research_study_observation(workspace, research_study):
    """Generate Observation summarizing research_study."""
    workspace_name = workspace.workspace.name
    sample_count = 0
    patient_count = 0
    blob_size_sum = 0
    missing_specimens_count = 0
    for p in workspace.patients.values():
        patient_count += 1
        if 'specimens' not in p:
            if missing_specimens_count == 0:
                logger.warning(('patient.specimens.missing', workspace_name, p['entityType'], p['name']))
            missing_specimens_count += 1
            continue
        for s in p['specimens']:
            sample_count += 1
            if 'tasks' not in s:
                logger.warning(('sample.tasks.missing', workspace_name, s['entityType'], s['name']))
                continue
            for t in s['tasks']:
                for property_blobs in t['outputs'].values():
                    for blob in property_blobs.values():
                        if 'size' not in blob:
                            logger.warning(('blob.attributes.missing', workspace_name, blob.keys(), blob))
                        blob_size_sum += int(blob.get('size', '0'))
    if missing_specimens_count > 0:
        logger.warning(('total.patient.specimens.missing', workspace_name, missing_specimens_count))

    consent_codes = "pending"
    primary_disease = "NA"
    study_design = "NA"
    data_type = "NA"

    if 'tracker' in workspace and workspace['tracker']:
        if "library:dataUseRestriction" in workspace['tracker']:
            consent_codes = workspace['tracker']['library:dataUseRestriction']

        assert 'library:indication' in workspace['tracker'], workspace['tracker']
        primary_disease = workspace['tracker']['library:indication']

        assert 'library:studyDesign' in workspace['tracker'], workspace['tracker']
        study_design = workspace['tracker']['library:studyDesign']

        assert 'library:datatype' in workspace['tracker'], workspace['tracker']
        data_type = workspace['tracker']['library:datatype']
    else:
        logger.error(('workspace.tracker.missing', workspace_name))

    observation = FHIRObservation.Observation(
        {
            'id': _id(workspace_name, 'Observation', research_study.id),
            "status": "final",
            'code': {'coding': [{'code': 'Summary', "display": "Variable Summary", "system": CANONICAL}]},
            'focus': [_ref(research_study).as_json()],
            'component': [
                {
                    'code': {'coding': [{'code': 'SampleCount', 'display': 'Number of Samples', "system": CANONICAL}]},
                    'valueInteger': sample_count
                },
                {
                    'code': {'coding': [{'code': 'Participant', 'display': 'Number of Participants', "system": CANONICAL}]},
                    'valueInteger': patient_count
                },
                {
                    'code': {'coding': [{'code': 'StorageSize', 'display': 'Size on Disk', "system": CANONICAL}]},
                    "valueQuantity": {
                        "value": blob_size_sum,
                        "system": "http://unitsofmeasure.org",
                        "code": "L"
                    }
                },
                {
                    'code': {'coding': [{'code': 'TerraWorkspace', 'display': 'Terra Workspace Name', "system": CANONICAL}]},
                    "valueString": workspace_name
                },
                {
                    'code': {'coding': [{'code': 'PrimaryDisease', 'display': 'Primary Disease', "system": CANONICAL}]},
                    "valueString": primary_disease
                },
                {
                    'code': {'coding': [{'code': 'StudyDesign', 'display': 'Study Design', "system": CANONICAL}]},
                    "valueString": study_design
                },
                {
                    'code': {'coding': [{'code': 'DataType', 'display': 'Data Type', "system": CANONICAL}]},
                    "valueString": data_type
                },
                {
                    'code': {'coding': [{'code': 'ConsentCodes', 'display': 'Consent Codes', "system": CANONICAL}]},
                    "valueString": consent_codes
                },

            ]
        }
    )

    if primary_disease and primary_disease != "NA":
        observation.component.append(
            {
                'code': {'coding': [{'code': 'PrimaryDiseaseOntology', 'display': 'Primary Disease Ontology', "system": CANONICAL}]},
                "valueCodeableConcept": {
                    "coding": [
                        _disease_coding(primary_disease)
                    ],
                    "text": primary_disease
                }
            }
        )

    return observation


def _terra_observation(workspace, entity, fhir_resource):
    """Create an observation with all the fields from the terra.workspace.patient"""
    assert 'attributes' in entity, ('no attributes on entity', entity)
    workspace_name = workspace.workspace.name

    def _str(val):
        """Return simple string or json dict."""
        if isinstance(val, dict):
            return json.dumps(val)
        return str(val)

    observation = FHIRObservation.Observation(
        {
            'id': _id(workspace_name, 'Observation', fhir_resource.id),
            "status": "final",
            'code': {'coding': [{'code': 'Detail', "display": f"{entity['entityType']} Detail", "system": "https://anvil.terra.bio/#workspaces/anvil-datastorage/"}]},
            'focus': [_ref(fhir_resource).as_json()],
            'component': [
                {
                    'code': {'coding': [{'code': 'Keys', "system": f"https://anvil.terra.bio/#workspaces/anvil-datastorage/{workspace_name}/{entity['entityType']}"}]},
                    'valueString': ','.join([key for key in entity['attributes']])
                },
                {
                    'code': {'coding': [{'code': 'Values', "system": f"https://anvil.terra.bio/#workspaces/anvil-datastorage/{workspace_name}/{entity['entityType']}/{entity['name']}"}]},
                    'valueString': ','.join([_str(val) for val in entity['attributes'].values()])
                },
            ]
        }
    )

    return observation


def _patient_reference(id):
    """Create a family member reference."""
    return FHIRReference.FHIRReference({'reference': f"Patient/{id}"})


def _generate_family_relationship_observations(workspace, family_id, family, config):
    """Create an observation of this patient's relationship to proband."""
    
    workspace_name = workspace.workspace.name
    assert list(config['mapping']['family_relationships'].keys()) == ['terms', 'intra_family']

    # local functions
    def _map_relationship(relationship):
        """Map term to standard concept."""
        assert list(config['mapping']['family_relationships']['terms'].keys()) == ['exact_match', 'occurrence'], config['mapping']['family_relationships']['terms'].keys()
        occurrences = config['mapping']['family_relationships']['terms']['occurrence']
        exact_matches = config['mapping']['family_relationships']['terms']['exact_match']
        # everything done in lower case
        relationship = relationship.lower().replace('_', ' ')
        # map variations of relationships to more standard term
        for occurrence in occurrences:
            if occurrence in relationship:
                return occurrences[occurrence]
        
        # look up term in mapping and return role code
        if relationship not in exact_matches:
            # logger.warning(('no.family.relationship.mapping', workspace_name, relationship, family))
            logger.warning(('no.family.relationship.mapping', relationship ))
            relationship = '_other_'
        return exact_matches[relationship]


    def _family_membership(mapped_family):
        """Yield family membership records based on relationship to proband."""
        assert list(config['mapping']['family_relationships']['intra_family'].keys()) == ['pairs', 'rules'], config['mapping']['family_relationships']['intra_family'].keys()
        flattened = []
        # family role is the RoleCode
        for family_role, _details in mapped_family.items():
            for detail in _details:
                detail['family_role'] = family_role
                flattened.append(detail)
        for focus, subject in itertools.permutations(flattened, 2):
            relationship = None
            roles = [focus['family_role'], subject['family_role']]
            pairs_results = {}
            pairs = config['mapping']['family_relationships']['intra_family']['pairs']
            for relationship in pairs:  
                pairs_results[relationship] = [pair == roles for pair in pairs[relationship] if pair == roles]
            relationship = next(iter([k  for k,v in pairs_results.items() if v]), None)
            if not relationship:
                rule_results = {}
                rules = config['mapping']['family_relationships']['intra_family']['rules']
                
                def _logging_eval(rule, locals):
                    """Log any exception."""
                    try:
                        return eval(rule, {'focus': focus, 'subject': subject})
                    except Exception as e:
                        logger.error((rule, e))
                        raise e

                for relationship in rules:
                    rule_results[relationship] = True in [_logging_eval(rule, {'focus': focus, 'subject': subject}) for rule in rules[relationship]]
                relationship = next(iter([k  for k,v in rule_results.items() if v]), None)
            if not relationship:
                logger.warning(('no.relationship.between.family.members', workspace_name, family_id, focus, subject))
            else:
                observation = {
                    'id': _id(workspace_name, 'Observation', 'family-relationship', family_id, relationship, detail['id']),
                    "meta": {
                        "profile": [
                            "https://ncpi-fhir.github.io/ncpi-fhir-ig/StructureDefinition/family-relationship"
                        ]
                    },
                    'identifier': [
                        _family_identifier(workspace, family_id).as_json(),
                        _patient_identifier(workspace, subject['name'], 'subject').as_json(),
                        _patient_identifier(workspace, focus['name'], 'focus').as_json()
                    ],
                    "status": "final",
                    'code': {'coding': [{'code': 'FAMMEMB', "display": "family member", "system": "http://terminology.hl7.org/CodeSystem/v3-RoleCode"}]},
                    'subject': _patient_reference(subject['id']).as_json(),  # this patient
                    'focus': [_patient_reference(focus['id']).as_json()],  # the proband
                    "valueCodeableConcept": {
                        "coding": [
                            {
                                "system": "http://terminology.hl7.org/CodeSystem/v3-RoleCode",
                                "code": relationship
                            }
                        ]
                    }
                }

                observation = FHIRObservation.Observation(observation)
                yield observation        
        # if no relationships
        yield from []



    def _map_relationship_OLD(relationship):
        """Map term to standard concept."""
        relationship = relationship.lower().replace('_', ' ')
        # https://terminology.hl7.org/3.1.0/CodeSystem-v3-RoleCode.html
        relationship_mapping = {
            'proband': 'ONESELF',
            'sister': 'SIS',
            'mother': 'MTH',
            'sibling': 'SIB',
            'affected father': 'FTH',
            'father': 'FTH',
            'brother': 'BRO',
            'paternal uncle': 'PUNCLE',
            'paternal aunt': 'PAUNT',
            'paternal grandfather': 'PGRFTH',
            'wife': 'WIFE',
            'husband': 'HUSB',
            'daughter': 'DAU',
            'son': 'SON',
            'parent': 'PRN',
            'child': 'CHILD',
            'uncle': 'UNCLE',
            'aunt': 'AUNT',
            'nephew or niece': 'NIENEPH',
            'brother or sister': 'SIB',
            'son or daughter': 'CHILD',
            'cousin': 'COUSN',
            'spouse': 'SPS',
            'brother or sister-in-law': 'INLAW',
            'grandfather': 'GRFTH',
            'grandmother': 'GRMTH',
            'inlaw': 'INLAW',
            'twin brother': 'TWINBRO',
            'twin sister': 'TWINSIS',
            'twin': 'TWIN',
            'maternal grandparent': 'MGRPRN',
            'paternal grandparent': 'PGRPRN',
            'half-sib': 'HSIB',
            'other': 'EXT',
            '_other_': 'EXT',
            '_not_related_': 'NOTRELATED'
        }
        if '#' in relationship and 'sibling' in relationship:
            relationship = 'sibling'
        if 'affected' == relationship:
            relationship = 'proband'
        if 'affected1' == relationship:
            relationship = 'proband'
        if 'affected2' == relationship:
            relationship = 'proband'
        if 'proband (twin)' == relationship:
            relationship = 'proband'
        if 'other' in relationship:
            relationship = 'other'
        if 'in law' in relationship:
            relationship = 'inlaw'
        if 'cousin' in relationship:
            relationship = 'cousin'
        if 'husband or wife' in relationship:
            relationship = 'spouse'
        if 'son or daughter' in relationship:
            relationship = 'sibling'
        if 'aunt or uncle' in relationship:
            relationship = 'other'
        if 'identical twin of proband' in relationship:
            relationship = 'twin'
        if 'father1' in relationship:
            relationship = 'father'
        if 'father2' in relationship:
            relationship = 'father'
        if 'sister2' in relationship:
            relationship = 'sister'
        if 'unaffected sibling' in relationship:
            relationship = 'sibling'
        if 'not related' in relationship:
            relationship = '_not_related_'
        if relationship not in relationship_mapping:
            logger.warning(('no.family.relationship.mapping', workspace_name, relationship, family))
            relationship = '_other_'
        return relationship_mapping[relationship]

    def _family_membership_OLD(mapped_family):
        """Yield family membership records based on relationship to proband."""
        flattened = []
        # family role is the RoleCode
        for family_role, _details in mapped_family.items():
            for detail in _details:
                detail['family_role'] = family_role
                flattened.append(detail)
        for focus, subject in itertools.permutations(flattened, 2):
            sorted_roles = sorted([focus['family_role'], subject['family_role']])
            relationship = None
            if sorted_roles == ['FTH', 'MTH']:
                relationship = 'SPS'  # spouse
            elif sorted_roles == ['SIB', 'SIB'] or sorted_roles == ['CHILD', 'CHILD']:
                relationship = 'SIB'  # sibling
            elif sorted_roles == ['CHILD', 'NIENEPH'] or sorted_roles == ['NIENEPH', 'SIB'] or sorted_roles == ['COUSN', 'SIB']:
                relationship = 'COUSN'  # cousin
            elif sorted_roles == ['NIENEPH', 'NIENEPH']:
                relationship = 'EXT'
            elif sorted_roles == ['MGRPRN', 'MGRPRN']:
                relationship = 'SPS'
            elif sorted_roles == ['PGRPRN', 'PGRPRN']:
                relationship = 'SPS'
            elif sorted_roles == ['MGRPRN', 'PGRPRN']:
                relationship = 'EXT'
            elif sorted_roles == ['COUSN', 'COUSN']:
                relationship = 'EXT'
            elif focus['family_role'] in ['FTH', 'MTH', 'PRN', 'SPS'] and subject['family_role'] in ['SIB', 'CHILD']:
                relationship = 'CHILD'
            elif subject['family_role'] in ['FTH', 'MTH', 'PRN', 'SPS'] and focus['family_role'] in ['SIB', 'CHILD']:
                relationship = 'PRN'  # parent
            elif 'EXT' in subject['family_role'] or 'EXT' in focus['family_role']:
                relationship = 'EXT'  # extended
            elif 'NIENEPH' in sorted_roles:
                relationship = 'EXT'
            elif 'MGRPRN' in sorted_roles:
                relationship = 'EXT'
            elif 'PGRPRN' in sorted_roles:
                relationship = 'EXT'
            elif 'COUSN' in sorted_roles:
                relationship = 'EXT'
            elif 'NOTRELATED' in sorted_roles:
                relationship = 'NOTRELATED'
            if not relationship:
                logger.warning(('no.relationship.between.family.members', workspace_name, family_id, focus, subject))
        yield from []

    def _proband_relationships(mapped_family):
        """Yield family membership records based on membership in a family."""
        for relationship, _details in mapped_family.items():
            for detail in _details:
                observation = {
                    'id': _id(workspace_name, 'Observation', 'family-relationship', family_id, relationship, detail['id']),
                    "meta": {
                        "profile": [
                            "https://ncpi-fhir.github.io/ncpi-fhir-ig/StructureDefinition/family-relationship"
                        ]
                    },
                    'identifier': [
                        _family_identifier(workspace, family_id).as_json(),
                        _patient_identifier(workspace, detail['name'], 'subject').as_json(),
                        _patient_identifier(workspace, mapped_family['ONESELF'][0]['name'], 'focus').as_json()
                    ],
                    "status": "final",
                    'code': {'coding': [{'code': 'FAMMEMB', "display": "family member", "system": "http://terminology.hl7.org/CodeSystem/v3-RoleCode"}]},
                    'subject': _patient_reference(detail['id']).as_json(),  # this patient
                    'focus': [_patient_reference(mapped_family['ONESELF'][0]['id']).as_json()],  # the proband
                    "valueCodeableConcept": {
                        "coding": [
                            {
                                "system": "http://terminology.hl7.org/CodeSystem/v3-RoleCode",
                                "code": relationship
                            }
                        ]
                    }
                }

                observation = FHIRObservation.Observation(observation)
                yield observation

    def _dispatcher():
        """Dispatch based on a) relationship to proband, or b) membership in family."""
        mapped_family = defaultdict(list)
        for relationship, details in family.items():
            mapped_family[_map_relationship(relationship)].extend(details)
        if 'ONESELF' not in mapped_family or len(mapped_family['ONESELF']) == 0:
            for obj in _family_membership(mapped_family):
                yield obj
        else:
            for obj in _proband_relationships(mapped_family):
                yield obj

    # main
    for _obj in _dispatcher():
        yield _obj


def _discover_families(workspace, patient, fhir_patient, families):
    """Populate families with relationships found in patients."""
    assert 'family_relationship_values' in patient
    for family_relationship in patient['family_relationship_values']:
        if family_relationship == {}:
            continue
        assert 'family_id' in family_relationship, ('no.id.in.family', workspace.workspace.name, family_relationship, patient)
        relationship = family_relationship.get('proband_relationship', family_relationship.get('family_relationship', None))

        # assert relationship, ('no.find.relationship', family_relationship, patient)
        if not relationship:
            relationship = 'proband'

        if relationship not in families[family_relationship['family_id']]:
            families[family_relationship['family_id']][relationship] = []

        family_member = {'id': fhir_patient.id, 'name': patient['name'], 'entityType': patient['entityType']}
        extra_fields = {k: v for k, v in family_relationship.items() if k not in ['family_id', 'family_relationship']}
        family_member.update(extra_fields)
        families[family_relationship['family_id']][relationship].append(family_member)


def generate_fhir(workspace, consortium_name, details, config):
    """Generate FHIR resources from normalized workspace."""

    (
        anvil_org,
        consortium_org,
        practitioner,
        data_store_org,
        workspace_org,
        practitioner_role,
        research_study
    ) = _create_administrative_entities(workspace, consortium_name)

    for resource in (
        anvil_org,
        consortium_org,
        practitioner,
        data_store_org,
        workspace_org,
        practitioner_role,
        research_study
    ):
        yield resource

    families = defaultdict(dict)
    for patient in workspace.patients.values():
        (fhir_patient, research_subject) = _create_individual(workspace, patient, workspace_org, research_study)
        yield fhir_patient
        yield research_subject

        for resource in _generate_specimen_descendants(workspace, patient, fhir_patient, details):
            yield resource

        if details:
            yield _terra_observation(workspace, patient, fhir_patient)

        _discover_families(workspace, patient, fhir_patient, families)

    yield _research_study_observation(workspace, research_study)

    for family_id, family in families.items():
        for family_relationship in _generate_family_relationship_observations(workspace, family_id, family, config):
            if family_relationship:
                yield family_relationship


def write(consortium_name, workspace, output_path, details, config):
    """Write normalized workspace to disk as FHIR."""
    emitters = {}
    import cProfile, pstats
    profiler = cProfile.Profile()
    profiler.enable()
    for fhir_resource in generate_fhir(workspace, consortium_name, details, config):
        resource_type = fhir_resource.resource_type
        data_store_name = ensure_data_store_name(workspace)
        dir_path = f"{output_path}/fhir/{data_store_name}/{consortium_name}/{workspace.workspace.name}"
        public_protected = 'protected'
        if resource_type in ['ResearchStudy', 'Organization', 'Practitioner', 'PractitionerRole']:
            public_protected = 'public'
        file_path = None

        resource_reference = None
        if hasattr(fhir_resource, 'focus') and fhir_resource.focus:
            resource_reference = [foci.reference for foci in fhir_resource.focus]
            if len(resource_reference) == 0:
                resource_reference = None
            else:
                resource_reference = resource_reference[0].split('/')[0]
            assert resource_reference

        resource_code = None
        if hasattr(fhir_resource, 'code') and fhir_resource.code:
            resource_code = [coding.code for coding in fhir_resource.code.coding]
            if len(resource_code) == 0:
                resource_code = None
            else:
                resource_code = resource_code[0]

        if resource_type == 'Observation':
            if resource_code == 'Summary':
                file_path = f"{dir_path}/public/{resource_reference}Observation{resource_code}.ndjson"
            else:
                file_path = f"{dir_path}/protected/{resource_reference}Observation{resource_code}.ndjson"

        if not file_path:
            file_path = f"{dir_path}/{public_protected}/{fhir_resource.relativeBase()}.ndjson"

        emitter = emitters.get(file_path, None)
        if emitter is None:
            os.makedirs(f"{dir_path}/public", exist_ok=True)
            os.makedirs(f"{dir_path}/protected", exist_ok=True)
            emitter = open(file_path, "w")
            logger.info(f"Writing {file_path}")
            emitters[file_path] = emitter
        json.dump(fhir_resource.as_json(), emitter, separators=(',', ':'))
        emitter.write('\n')
    profiler.disable()
    # Export profiler output to file
    stats = pstats.Stats(profiler)
    stats.dump_stats(f'{output_path}/program.prof')